"""
简化的 Memory 类，提供更友好的用户接口

提供简化的 API，让用户可以直接使用 `memory = Memory()` 然后调用 `memory.search()` 和 `memory.add()`
"""

import os
from typing import Optional, Dict, Any, List, Union
from .sync_client import TiMEMClient
from .exceptions import TiMEMError


class Memory:
    """
    简化的记忆管理类
    
    提供更简洁的 API 接口，隐藏底层的复杂性。
    
    Example:
        ```python
        memory = Memory()
        
        # 搜索相关记忆
        results = memory.search(query="用户问题", user_id="user123", limit=3)
        
        # 添加对话记忆
        messages = [
            {"role": "user", "content": "你好"},
            {"role": "assistant", "content": "您好！有什么可以帮您的吗？"}
        ]
        memory.add(messages, user_id="user123")
        ```
    """
    
    def __init__(
        self,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        **kwargs
    ):
        """
        初始化 Memory 实例
        
        Args:
            api_key: TiMEM API 密钥。如果不提供，从环境变量 TIMEM_API_KEY 读取
            base_url: API 基础 URL。如果不提供，从环境变量 TIMEM_BASE_URL 读取，默认为 "http://localhost:8001"
            **kwargs: 其他传递给 TiMEMClient 的参数
        """
        # 从环境变量读取配置，如果没有提供
        api_key = api_key or os.getenv("TIMEM_API_KEY", "")
        base_url = base_url or os.getenv("TIMEM_BASE_URL", "http://localhost:8001")
        
        if not api_key:
            raise ValueError("api_key 必须提供，可以通过参数传入或设置环境变量 TIMEM_API_KEY")
        
        self._client = TiMEMClient(
            api_key=api_key,
            base_url=base_url,
            **kwargs
        )
    
    def _to_user_id_int(self, user_id: Union[str, int]) -> Optional[int]:
        """
        将 user_id 转换为整数
        
        如果 user_id 是字符串：
        - 如果是纯数字字符串，转换为整数
        - 否则，使用 hash 生成稳定的整数ID（取绝对值确保为正数）
        
        Args:
            user_id: 用户ID（字符串或整数）
        
        Returns:
            整数用户ID，如果无法转换则返回 None
        """
        if isinstance(user_id, int):
            return user_id
        
        if isinstance(user_id, str):
            # 尝试直接转换为整数
            if user_id.isdigit():
                return int(user_id)
            # 对于非数字字符串，使用 hash 生成稳定的整数ID
            # 使用 abs 确保为正数，避免负数ID问题
            return abs(hash(user_id)) % (10 ** 9)  # 限制在合理范围内
        
        return None
    
    def search(
        self,
        query: str,
        user_id: Union[str, int] = "default_user",
        limit: int = 10,
        character_id: Optional[str] = None,
        include_context: bool = False
    ) -> Dict[str, Any]:
        """
        搜索相关记忆 - 使用智能语义搜索
        
        Args:
            query: 搜索查询文本（语义搜索）
            user_id: 用户ID，可以是字符串或整数
            limit: 返回结果数量限制（注意：实际返回数量可能受 score_threshold 影响）
            character_id: 角色ID（可选），用于过滤特定角色的记忆
            include_context: 是否包含上下文信息（session_id, character_id等）
        
        Returns:
            包含搜索结果的字典，格式：
            {
                "results": [
                    {
                        "memory": "记忆内容",
                        "score": 0.95,
                        "id": "记忆ID",
                        "layer": "L1",
                        "metadata": {...},
                        "created_at": "...",
                        ...
                    },
                    ...
                ],
                "total": 10,
                "query": "查询文本"
            }
        """
        # 转换 user_id 为字符串（新API使用字符串类型）
        user_id_str = str(user_id) if user_id else None
        
        try:
            # 调用新的智能搜索API
            result = self._client.search_memories(
                query_text=query,
                user_id=user_id_str,
                character_id=character_id,
                include_context=include_context,
                format="simple",
                search_mode="enhanced_semantic",
                score_threshold=0.5
            )
            
            # 格式化返回结果，匹配用户期望的格式
            formatted_results = []
            memories = result.get("memories", [])
            
            # 如果指定了limit，只取前limit个结果
            if limit and limit > 0:
                memories = memories[:limit]
            
            for mem in memories:
                # 提取记忆内容
                memory_text = mem.get("memory", mem.get("content", ""))
                if not memory_text and isinstance(mem.get("data"), dict):
                    memory_text = mem["data"].get("memory", "")
                
                # 提取相似度分数
                metadata = mem.get("metadata", {})
                score = metadata.get("score", mem.get("score", 0.0))
                
                formatted_results.append({
                    "memory": memory_text,
                    "score": score,
                    "id": mem.get("id"),
                    "layer": mem.get("layer"),
                    "metadata": metadata,
                    "created_at": mem.get("created_at"),
                    "updated_at": mem.get("updated_at"),
                    **{k: v for k, v in mem.items() if k not in [
                        "memory", "content", "score", "id", "layer", 
                        "metadata", "created_at", "updated_at", "data"
                    ]}
                })
            
            return {
                "results": formatted_results,
                "total": result.get("total", len(formatted_results)),
                "query": query
            }
        except Exception as e:
            raise TiMEMError(f"搜索记忆失败: {str(e)}") from e
    
    def add(
        self,
        messages: List[Dict[str, str]],
        user_id: Union[str, int] = "default_user",
        character_id: Optional[str] = None,
        session_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        添加对话记忆 - 使用智能记忆生成
        
        从对话消息中自动提取并生成记忆。
        
        Args:
            messages: 对话消息列表，格式：[{"role": "user", "content": "..."}, ...]
            user_id: 用户ID，可以是字符串或整数
            character_id: 角色ID（expert_id），可选
            session_id: 会话ID，如果不提供则自动生成
        
        Returns:
            添加结果字典，格式：
            {
                "success": True,
                "memories": [
                    {
                        "id": "记忆ID",
                        "data": {"memory": "记忆内容"},
                        "event": "ADD"
                    },
                    ...
                ],
                "total": 1
            }
        """
        if not messages:
            raise ValueError("messages 不能为空")
        
        if not character_id:
            raise ValueError("character_id 必须提供")
        
        # 转换 user_id 为字符串（新API使用字符串类型）
        user_id_str = str(user_id) if user_id else None
        
        # 如果没有提供 session_id，生成一个基于 user_id 的默认值
        if not session_id:
            import hashlib
            session_id = f"session_{hashlib.md5(str(user_id).encode()).hexdigest()[:8]}"
        
        try:
            # 调用新的生成记忆API
            result = self._client.generate_memory(
                character_id=character_id,
                session_id=session_id,
                messages=messages,
                user_id=user_id_str,
                format="compact"
            )
            
            # 处理返回结果
            if isinstance(result, list):
                # compact格式返回数组
                memory_ids = []
                for item in result:
                    memory_id = item.get("id")
                    if memory_id:
                        memory_ids.append(memory_id)
                
                return {
                    "success": True,
                    "memories": result,
                    "memory_id": memory_ids[0] if memory_ids else None,
                    "memory_ids": memory_ids,
                    "total": len(result),
                    "message": f"成功生成 {len(result)} 条记忆"
                }
            else:
                # 兼容其他格式
                return {
                    "success": True,
                    "memories": [result] if result else [],
                    "memory_id": result.get("id") if isinstance(result, dict) else None,
                    "total": 1 if result else 0,
                    "message": "记忆添加成功"
                }
        except Exception as e:
            raise TiMEMError(f"添加记忆失败: {str(e)}") from e
    
    def close(self):
        """
        关闭客户端连接
        
        清理资源，关闭连接池和事件循环等。
        """
        if hasattr(self, '_client') and self._client:
            try:
                self._client.close()
            except Exception:
                pass
    
    def __enter__(self):
        """上下文管理器入口"""
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器出口"""
        self.close()




"""
TiMEM熔断器

提供企业级熔断器功能，支持自动故障检测、熔断恢复、降级策略等。
"""

import asyncio
import time
import logging
from typing import Dict, Any, Optional, Callable, Union
from dataclasses import dataclass
from enum import Enum
from .exceptions import TiMEMError, CircuitBreakerError


class CircuitState(Enum):
    """熔断器状态"""
    CLOSED = "closed"      # 正常状态
    OPEN = "open"          # 熔断状态
    HALF_OPEN = "half_open"  # 半开状态


@dataclass
class CircuitBreakerConfig:
    """熔断器配置"""
    failure_threshold: int = 5          # 失败阈值
    recovery_timeout: float = 60.0      # 恢复超时时间（秒）
    success_threshold: int = 3          # 半开状态下的成功阈值
    timeout: float = 30.0               # 请求超时时间
    max_requests: int = 100             # 最大请求数（用于统计）
    error_rate_threshold: float = 0.5   # 错误率阈值


class CircuitBreaker:
    """熔断器实现"""
    
    def __init__(
        self,
        name: str,
        config: Optional[CircuitBreakerConfig] = None,
        fallback_func: Optional[Callable] = None
    ):
        self.name = name
        self.config = config or CircuitBreakerConfig()
        self.fallback_func = fallback_func
        
        # 状态管理
        self.state = CircuitState.CLOSED
        self.failure_count = 0
        self.success_count = 0
        self.last_failure_time = 0
        self.request_count = 0
        self.error_count = 0
        
        # 日志记录器
        self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}.{name}")
        
        # 统计信息
        self.stats = {
            "total_requests": 0,
            "successful_requests": 0,
            "failed_requests": 0,
            "circuit_opened_count": 0,
            "fallback_calls": 0
        }
    
    def _should_attempt_reset(self) -> bool:
        """检查是否应该尝试重置熔断器"""
        if self.state != CircuitState.OPEN:
            return False
        
        return time.time() - self.last_failure_time >= self.config.recovery_timeout
    
    def _on_success(self):
        """处理成功请求"""
        self.failure_count = 0
        self.success_count += 1
        self.stats["successful_requests"] += 1
        
        if self.state == CircuitState.HALF_OPEN:
            if self.success_count >= self.config.success_threshold:
                self.state = CircuitState.CLOSED
                self.success_count = 0
                self.logger.info(f"熔断器 {self.name} 已恢复为正常状态")
    
    def _on_failure(self):
        """处理失败请求"""
        self.failure_count += 1
        self.error_count += 1
        self.last_failure_time = time.time()
        self.stats["failed_requests"] += 1
        
        if self.state == CircuitState.CLOSED:
            if self.failure_count >= self.config.failure_threshold:
                self.state = CircuitState.OPEN
                self.stats["circuit_opened_count"] += 1
                self.logger.warning(f"熔断器 {self.name} 已打开，失败次数: {self.failure_count}")
        elif self.state == CircuitState.HALF_OPEN:
            self.state = CircuitState.OPEN
            self.logger.warning(f"熔断器 {self.name} 在半开状态下失败，重新打开")
    
    def _get_error_rate(self) -> float:
        """获取错误率"""
        if self.request_count == 0:
            return 0.0
        return self.error_count / self.request_count
    
    def _should_open_circuit(self) -> bool:
        """检查是否应该打开熔断器"""
        # 基于失败次数
        if self.failure_count >= self.config.failure_threshold:
            return True
        
        # 基于错误率
        if (self.request_count >= self.config.max_requests and 
            self._get_error_rate() >= self.config.error_rate_threshold):
            return True
        
        return False
    
    async def call(self, func: Callable, *args, **kwargs) -> Any:
        """执行函数调用，带熔断保护"""
        self.stats["total_requests"] += 1
        self.request_count += 1
        
        # 检查熔断器状态
        if self.state == CircuitState.OPEN:
            if not self._should_attempt_reset():
                # 调用降级函数
                if self.fallback_func:
                    self.stats["fallback_calls"] += 1
                    self.logger.info(f"熔断器 {self.name} 处于打开状态，调用降级函数")
                    return await self._call_fallback(*args, **kwargs)
                else:
                    raise CircuitBreakerError(f"熔断器 {self.name} 处于打开状态")
            else:
                # 尝试重置为半开状态
                self.state = CircuitState.HALF_OPEN
                self.success_count = 0
                self.logger.info(f"熔断器 {self.name} 尝试重置为半开状态")
        
        # 执行函数调用
        try:
            start_time = time.time()
            
            # 设置超时
            if asyncio.iscoroutinefunction(func):
                result = await asyncio.wait_for(
                    func(*args, **kwargs),
                    timeout=self.config.timeout
                )
            else:
                result = await asyncio.wait_for(
                    asyncio.get_event_loop().run_in_executor(None, func, *args, **kwargs),
                    timeout=self.config.timeout
                )
            
            # 记录成功
            self._on_success()
            
            duration = time.time() - start_time
            self.logger.debug(f"熔断器 {self.name} 调用成功: {duration:.2f}s")
            
            return result
            
        except asyncio.TimeoutError:
            self.logger.warning(f"熔断器 {self.name} 调用超时")
            self._on_failure()
            raise CircuitBreakerError(f"熔断器 {self.name} 调用超时")
            
        except Exception as e:
            self.logger.warning(f"熔断器 {self.name} 调用失败: {str(e)}")
            self._on_failure()
            
            # 检查是否应该打开熔断器
            if self._should_open_circuit():
                self.state = CircuitState.OPEN
                self.stats["circuit_opened_count"] += 1
                self.logger.warning(f"熔断器 {self.name} 因错误率过高而打开")
            
            # 如果是熔断器打开状态，调用降级函数
            if self.state == CircuitState.OPEN and self.fallback_func:
                self.stats["fallback_calls"] += 1
                self.logger.info(f"熔断器 {self.name} 调用降级函数")
                return await self._call_fallback(*args, **kwargs)
            
            raise
    
    async def _call_fallback(self, *args, **kwargs) -> Any:
        """调用降级函数"""
        if not self.fallback_func:
            raise CircuitBreakerError(f"熔断器 {self.name} 没有配置降级函数")
        
        try:
            if asyncio.iscoroutinefunction(self.fallback_func):
                return await self.fallback_func(*args, **kwargs)
            else:
                return await asyncio.get_event_loop().run_in_executor(
                    None, self.fallback_func, *args, **kwargs
                )
        except Exception as e:
            self.logger.error(f"降级函数调用失败: {str(e)}")
            raise CircuitBreakerError(f"降级函数调用失败: {str(e)}")
    
    def get_state(self) -> Dict[str, Any]:
        """获取熔断器状态"""
        return {
            "name": self.name,
            "state": self.state.value,
            "failure_count": self.failure_count,
            "success_count": self.success_count,
            "request_count": self.request_count,
            "error_count": self.error_count,
            "error_rate": self._get_error_rate(),
            "last_failure_time": self.last_failure_time,
            "stats": self.stats.copy(),
            "config": {
                "failure_threshold": self.config.failure_threshold,
                "recovery_timeout": self.config.recovery_timeout,
                "success_threshold": self.config.success_threshold,
                "timeout": self.config.timeout,
                "max_requests": self.config.max_requests,
                "error_rate_threshold": self.config.error_rate_threshold
            }
        }
    
    def reset(self):
        """重置熔断器"""
        self.state = CircuitState.CLOSED
        self.failure_count = 0
        self.success_count = 0
        self.error_count = 0
        self.last_failure_time = 0
        self.logger.info(f"熔断器 {self.name} 已重置")


class CircuitBreakerManager:
    """熔断器管理器"""
    
    def __init__(self):
        self._breakers: Dict[str, CircuitBreaker] = {}
        self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
    
    def get_breaker(
        self,
        name: str,
        config: Optional[CircuitBreakerConfig] = None,
        fallback_func: Optional[Callable] = None
    ) -> CircuitBreaker:
        """获取或创建熔断器"""
        if name not in self._breakers:
            self._breakers[name] = CircuitBreaker(
                name=name,
                config=config,
                fallback_func=fallback_func
            )
            self.logger.info(f"创建熔断器: {name}")
        
        return self._breakers[name]
    
    def get_all_states(self) -> Dict[str, Dict[str, Any]]:
        """获取所有熔断器状态"""
        return {
            name: breaker.get_state()
            for name, breaker in self._breakers.items()
        }
    
    def reset_all(self):
        """重置所有熔断器"""
        for breaker in self._breakers.values():
            breaker.reset()
        self.logger.info("所有熔断器已重置")
    
    def remove_breaker(self, name: str):
        """移除熔断器"""
        if name in self._breakers:
            del self._breakers[name]
            self.logger.info(f"移除熔断器: {name}")


# 全局熔断器管理器实例
_global_breaker_manager = CircuitBreakerManager()


def get_circuit_breaker(
    name: str,
    config: Optional[CircuitBreakerConfig] = None,
    fallback_func: Optional[Callable] = None
) -> CircuitBreaker:
    """获取全局熔断器"""
    return _global_breaker_manager.get_breaker(
        name=name,
        config=config,
        fallback_func=fallback_func
    )


def get_all_breaker_states() -> Dict[str, Dict[str, Any]]:
    """获取所有熔断器状态"""
    return _global_breaker_manager.get_all_states()


def reset_all_breakers():
    """重置所有熔断器"""
    _global_breaker_manager.reset_all()



"""
TiMEM SDK Exceptions

Custom exception classes for the TiMEM SDK.
"""


class TiMEMError(Exception):
    """Base exception for TiMEM SDK."""
    pass


class AuthenticationError(TiMEMError):
    """Raised when authentication fails."""
    pass


class APIError(TiMEMError):
    """Raised when API returns an error."""
    
    def __init__(self, message: str, status_code: int = None):
        super().__init__(message)
        self.status_code = status_code


class ValidationError(TiMEMError):
    """Raised when request validation fails."""
    pass


class ConnectionError(TiMEMError):
    """Raised when connection to TiMEM server fails."""
    pass


class TimeoutError(TiMEMError):
    """Raised when request times out."""
    pass


class CircuitBreakerError(TiMEMError):
    """Raised when circuit breaker is open."""
    pass


class RateLimitError(TiMEMError):
    """Raised when rate limit is exceeded."""
    pass


from typing import Dict, Any, Optional, List, AsyncGenerator
from ..types.optimization import OptimizationRequest, OptimizationResponse

class Optimization:
    """优化相关API"""
    
    def __init__(self, client):
        self.client = client
    
    async def optimize(
        self,
        request: OptimizationRequest
    ) -> OptimizationResponse:
        """
        优化简历内容
        
        Args:
            request: 优化请求
            
        Returns:
            优化结果
        """
        data = request.dict()
        response = await self.client._request("POST", "/optimization/optimize", data)
        return OptimizationResponse(**response)
    
    async def get_optimization_result(self, optimization_id: str) -> OptimizationResponse:
        """
        获取优化结果
        
        Args:
            optimization_id: 优化ID
            
        Returns:
            优化结果
        """
        response = await self.client._request("GET", f"/optimization/{optimization_id}")
        return OptimizationResponse(**response)
    
    async def stream_optimization(
        self,
        request: OptimizationRequest
    ) -> AsyncGenerator[Dict[str, Any], None]:
        """
        流式优化
        
        Args:
            request: 优化请求
            
        Yields:
            优化进度更新
        """
        data = request.dict()
        
        async with self.client._client.stream(
            "POST",
            f"{self.client.api_base}/optimization/stream",
            json=data
        ) as response:
            async for line in response.aiter_lines():
                if line.startswith("data: "):
                    try:
                        yield json.loads(line[6:])
                    except json.JSONDecodeError:
                        continue



"""
TiMEM增强异步客户端

集成连接池、熔断器、监控等企业级特性的异步客户端。
"""

import asyncio
import time
import logging
from typing import Optional, Dict, Any, List
from .exceptions import TiMEMError, AuthenticationError, APIError, ValidationError, ConnectionError, CircuitBreakerError
from .connection_pool import ConnectionPool, ConnectionConfig, get_connection_pool
from .circuit_breaker import CircuitBreaker, CircuitBreakerConfig, get_circuit_breaker
from .monitoring import PerformanceMonitor, get_global_monitor


class EnhancedAsyncTiMEMClient:
    """
    增强的异步TiMEM客户端
    
    集成企业级特性：
    - 连接池管理
    - 熔断器保护
    - 性能监控
    - 结构化日志
    - 自动重试
    """
    
    def __init__(
        self,
        api_key: str,
        base_url: str = "http://localhost:8001",
        connection_config: Optional[ConnectionConfig] = None,
        circuit_breaker_config: Optional[CircuitBreakerConfig] = None,
        enable_monitoring: bool = True,
        verify_ssl: bool = False,
        **kwargs
    ):
        """
        初始化增强的异步TiMEM客户端
        
        Args:
            api_key: TiMEM API密钥
            base_url: API基础URL
            connection_config: 连接池配置
            circuit_breaker_config: 熔断器配置
            enable_monitoring: 是否启用监控
            verify_ssl: 是否验证SSL证书
            **kwargs: 其他参数
        """
        if not api_key:
            raise ValidationError("API key is required")
        
        self.api_key = api_key
        self.base_url = base_url
        self.verify_ssl = verify_ssl
        
        # 连接池配置
        self.connection_config = connection_config or ConnectionConfig()
        
        # 熔断器配置
        self.circuit_breaker_config = circuit_breaker_config or CircuitBreakerConfig()
        
        # 监控配置
        self.enable_monitoring = enable_monitoring
        self.monitor = get_global_monitor() if enable_monitoring else None
        
        # 日志记录器
        self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
        
        # 连接池和熔断器（延迟初始化）
        self._pool: Optional[ConnectionPool] = None
        self._circuit_breaker: Optional[CircuitBreaker] = None
        
        # 统计信息
        self.stats = {
            "total_requests": 0,
            "successful_requests": 0,
            "failed_requests": 0,
            "circuit_breaker_trips": 0,
            "connection_pool_resets": 0
        }
    
    async def __aenter__(self):
        """异步上下文管理器入口"""
        await self.initialize()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器出口"""
        await self.close()
    
    async def initialize(self):
        """初始化客户端"""
        try:
            # 初始化连接池
            self._pool = await get_connection_pool(
                base_url=self.base_url,
                api_key=self.api_key,
                config=self.connection_config,
                verify_ssl=self.verify_ssl
            )
            
            # 初始化熔断器
            self._circuit_breaker = get_circuit_breaker(
                name=f"timem_{self.base_url}",
                config=self.circuit_breaker_config,
                fallback_func=self._fallback_handler
            )
            
            self.logger.info(f"增强TiMEM客户端已初始化: {self.base_url}")
            
        except Exception as e:
            self.logger.error(f"客户端初始化失败: {str(e)}")
            raise ConnectionError(f"Failed to initialize client: {str(e)}")
    
    async def close(self):
        """关闭客户端"""
        if self._pool:
            await self._pool.close()
            self._pool = None
        
        self.logger.info("增强TiMEM客户端已关闭")
    
    async def _fallback_handler(self, *args, **kwargs) -> Dict[str, Any]:
        """降级处理函数"""
        self.stats["circuit_breaker_trips"] += 1
        self.logger.warning("熔断器触发，使用降级处理")
        
        return {
            "success": False,
            "error": "Service temporarily unavailable",
            "fallback": True,
            "message": "TiMEM服务暂时不可用，请稍后重试"
        }
    
    async def _make_enhanced_request(
        self,
        method: str,
        endpoint: str,
        data: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
        user_id: Optional[str] = None,
        domain: Optional[str] = None,
        **kwargs
    ) -> Dict[str, Any]:
        """
        发送增强请求
        
        集成连接池、熔断器、监控等功能
        """
        if not self._pool:
            await self.initialize()
        
        start_time = time.time()
        success = False
        error_message = None
        status_code = 200
        
        try:
            # 使用熔断器保护
            result = await self._circuit_breaker.call(
                self._pool.make_request,
                method=method,
                endpoint=endpoint,
                data=data,
                params=params,
                **kwargs
            )
            
            success = True
            self.stats["successful_requests"] += 1
            
            return result
            
        except CircuitBreakerError as e:
            error_message = f"Circuit breaker open: {str(e)}"
            status_code = 503
            self.stats["circuit_breaker_trips"] += 1
            raise
            
        except (ConnectionError, APIError, AuthenticationError) as e:
            error_message = str(e)
            status_code = getattr(e, 'status_code', 500)
            self.stats["failed_requests"] += 1
            raise
            
        except Exception as e:
            error_message = str(e)
            status_code = 500
            self.stats["failed_requests"] += 1
            raise TiMEMError(f"Request failed: {str(e)}")
            
        finally:
            # 记录统计信息
            self.stats["total_requests"] += 1
            duration = time.time() - start_time
            
            # 记录监控数据
            if self.monitor:
                self.monitor.record_request(
                    method=method,
                    endpoint=endpoint,
                    duration=duration,
                    status_code=status_code,
                    success=success,
                    user_id=user_id,
                    domain=domain,
                    error_message=error_message
                )
    
    # ==================== Experience Learning Engine APIs ====================
    
    async def learn(
        self,
        domain: str = "general",
        feedback_cases: Optional[List[Dict[str, Any]]] = None,
        min_case_count: int = 3,
        min_adoption_rate: float = 0.6,
        min_confidence_score: float = 0.5,
        strategy: str = "adaptive",
        user_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        学习生成经验规则（增强版）
        """
        if not domain:
            raise ValidationError("Domain is required")
        
        # 如果提供了反馈案例，先收集
        if feedback_cases:
            import datetime
            batch_id = f"batch_{datetime.datetime.now().strftime('%Y%m%d_%H%M%S')}"
            
            collect_data = {
                "batch_id": batch_id,
                "feedback_cases": feedback_cases
            }
            
            # 收集反馈
            await self._make_enhanced_request(
                "POST", "/api/v1/rules/feedback/collect",
                data=collect_data, user_id=user_id, domain=domain
            )
        
        # 触发学习
        data = {
            "domain": domain,
            "min_case_count": min_case_count,
            "min_adoption_rate": min_adoption_rate,
            "min_confidence_score": min_confidence_score
        }
        
        params = {"summarization_strategy": strategy}
        
        return await self._make_enhanced_request(
            "POST", "/api/v1/rules/learn-with-llm",
            data=data, params=params, user_id=user_id, domain=domain
        )
    
    async def recall(
        self,
        context: Dict[str, Any],
        domain: str = "general",
        top_k: int = 5,
        min_confidence: float = 0.5,
        user_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        召回相关经验规则（增强版）
        """
        if not context:
            raise ValidationError("Context is required")
        
        data = {
            "domain": domain,
            "context": context,
            "top_k": top_k,
            "min_confidence": min_confidence
        }
        
        return await self._make_enhanced_request(
            "POST", "/api/v1/rules/rule-call",
            data=data, user_id=user_id, domain=domain
        )
    
    # ==================== Memory Management APIs ====================
    
    async def add_memory(
        self,
        user_id: int,
        domain: str,
        content: Dict[str, Any],
        layer_type: str = "L1",
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        添加记忆（增强版）
        """
        if not content:
            raise ValidationError("Memory content is required")
        
        data = {
            "user_id": user_id,
            "domain": domain,
            "content": content,
            "layer_type": layer_type,
            "tags": tags or [],
            "keywords": keywords or []
        }
        
        return await self._make_enhanced_request(
            "POST", "/api/v1/memory/memories",
            data=data, user_id=str(user_id), domain=domain
        )
    
    async def search_memory(
        self,
        user_id: Optional[int] = None,
        domain: Optional[str] = None,
        layer_type: Optional[str] = None,
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        limit: int = 20,
        offset: int = 0
    ) -> Dict[str, Any]:
        """
        搜索记忆（增强版）
        """
        params = {
            "limit": limit,
            "offset": offset
        }
        
        if user_id:
            params["user_id"] = user_id
        if domain:
            params["domain"] = domain
        if layer_type:
            params["layer_type"] = layer_type
        if tags:
            params["tags"] = ",".join(tags)
        if keywords:
            params["keywords"] = ",".join(keywords)
        
        return await self._make_enhanced_request(
            "GET", "/api/v1/memory/memories",
            params=params, user_id=str(user_id) if user_id else None, domain=domain
        )
    
    # ==================== 批量操作 ====================
    
    async def batch_learn(
        self,
        domains: List[str],
        min_case_count: int = 3,
        min_adoption_rate: float = 0.6,
        min_confidence_score: float = 0.5,
        strategy: str = "adaptive"
    ) -> List[Dict[str, Any]]:
        """
        批量学习（增强版）
        """
        if not domains:
            return []
        
        # 创建并发任务
        tasks = []
        for domain in domains:
            task = self.learn(
                domain=domain,
                min_case_count=min_case_count,
                min_adoption_rate=min_adoption_rate,
                min_confidence_score=min_confidence_score,
                strategy=strategy
            )
            tasks.append(task)
        
        # 并发执行
        try:
            results = await asyncio.gather(*tasks, return_exceptions=True)
            
            # 处理结果
            processed_results = []
            for i, result in enumerate(results):
                if isinstance(result, Exception):
                    processed_results.append({
                        'error': str(result),
                        'success': False,
                        'domain': domains[i]
                    })
                else:
                    processed_results.append({
                        'result': result,
                        'success': True,
                        'domain': domains[i]
                    })
            
            return processed_results
            
        except Exception as e:
            raise TiMEMError(f"Batch learn failed: {str(e)}")
    
    # ==================== 系统监控 ====================
    
    async def get_health_status(self) -> Dict[str, Any]:
        """
        获取健康状态（增强版）
        """
        try:
            # 基础健康检查
            health_data = await self._make_enhanced_request("GET", "/api/v1/health")
            
            # 添加增强信息
            enhanced_health = {
                **health_data,
                "enhanced_features": {
                    "connection_pool": self._pool.get_stats() if self._pool else None,
                    "circuit_breaker": self._circuit_breaker.get_state() if self._circuit_breaker else None,
                    "monitoring": self.monitor.get_health_status() if self.monitor else None,
                    "client_stats": self.stats.copy()
                }
            }
            
            return enhanced_health
            
        except Exception as e:
            return {
                "status": "unhealthy",
                "error": str(e),
                "enhanced_features": {
                    "connection_pool": self._pool.get_stats() if self._pool else None,
                    "circuit_breaker": self._circuit_breaker.get_state() if self._circuit_breaker else None,
                    "monitoring": self.monitor.get_health_status() if self.monitor else None,
                    "client_stats": self.stats.copy()
                }
            }
    
    def get_client_stats(self) -> Dict[str, Any]:
        """获取客户端统计信息"""
        return {
            "client_stats": self.stats.copy(),
            "connection_pool": self._pool.get_stats() if self._pool else None,
            "circuit_breaker": self._circuit_breaker.get_state() if self._circuit_breaker else None,
            "monitoring": self.monitor.get_health_status() if self.monitor else None
        }
    
    async def reset_connections(self):
        """重置连接"""
        if self._pool:
            await self._pool.reset()
            self.stats["connection_pool_resets"] += 1
            self.logger.info("连接池已重置")
    
    def reset_circuit_breaker(self):
        """重置熔断器"""
        if self._circuit_breaker:
            self._circuit_breaker.reset()
            self.logger.info("熔断器已重置")


# 便利函数
async def create_enhanced_client(
    api_key: str,
    base_url: str = "http://localhost:8001",
    connection_config: Optional[ConnectionConfig] = None,
    circuit_breaker_config: Optional[CircuitBreakerConfig] = None,
    enable_monitoring: bool = True,
    verify_ssl: bool = False,
    **kwargs
) -> EnhancedAsyncTiMEMClient:
    """
    创建增强的异步TiMEM客户端
    
    Args:
        api_key: TiMEM API密钥
        base_url: API基础URL
        connection_config: 连接池配置
        circuit_breaker_config: 熔断器配置
        enable_monitoring: 是否启用监控
        verify_ssl: 是否验证SSL证书
        **kwargs: 其他参数
    
    Returns:
        增强的异步TiMEM客户端
    """
    client = EnhancedAsyncTiMEMClient(
        api_key=api_key,
        base_url=base_url,
        connection_config=connection_config,
        circuit_breaker_config=circuit_breaker_config,
        enable_monitoring=enable_monitoring,
        verify_ssl=verify_ssl,
        **kwargs
    )
    
    await client.initialize()
    return client



"""
TiMEM同步客户端（兼容层）

为需要同步调用的用户提供兼容层，内部使用异步实现。
"""

import asyncio
from typing import Optional, Dict, Any, List
from .async_client import TiMEMClient as AsyncTiMEMClient
from .exceptions import TiMEMError, AuthenticationError, APIError, ValidationError


class TiMEMClient:
    """
    TiMEM同步客户端（兼容层）
    
    内部使用异步实现，对外提供同步接口。
    适用于需要同步调用的场景。
    """
    
    def __init__(
        self,
        api_key: str,
        base_url: str = "http://localhost:8001",
        timeout: float = 60.0,
        max_retries: int = 3,
        retry_delay: float = 1.0,
        verify_ssl: bool = False,
        enable_connection_pool: bool = True,
        enable_circuit_breaker: bool = True,
        enable_monitoring: bool = True,
        **kwargs
    ):
        """
        初始化同步TiMEM客户端
        
        Args:
            api_key: TiMEM API密钥
            base_url: API基础URL
            timeout: 请求超时时间
            max_retries: 最大重试次数
            retry_delay: 重试延迟
            verify_ssl: 是否验证SSL证书
            enable_connection_pool: 启用连接池
            enable_circuit_breaker: 启用熔断器
            enable_monitoring: 启用监控
            **kwargs: 其他参数
        """
        self._async_client = AsyncTiMEMClient(
            api_key=api_key,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries,
            retry_delay=retry_delay,
            verify_ssl=verify_ssl,
            enable_connection_pool=enable_connection_pool,
            enable_circuit_breaker=enable_circuit_breaker,
            enable_monitoring=enable_monitoring,
            **kwargs
        )
        self._loop = None
        self._loop_owner = False  # 标记是否拥有事件循环
    
    def __enter__(self):
        """同步上下文管理器入口"""
        self._loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self._loop)
        self._loop_owner = True
        self._loop.run_until_complete(self._async_client.__aenter__())
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        """同步上下文管理器出口"""
        if self._loop:
            self._loop.run_until_complete(self._async_client.__aexit__(exc_type, exc_val, exc_tb))
            self._loop.close()
            self._loop = None
            self._loop_owner = False
    
    def _run_async(self, coro):
        """运行异步协程"""
        # 检查是否有可用的事件循环
        try:
            loop = asyncio.get_event_loop()
            if loop.is_closed():
                # 如果事件循环已关闭，创建新的
                loop = None
        except RuntimeError:
            # 如果没有事件循环，创建新的
            loop = None
        
        # 如果没有可用的事件循环，创建新的并保存
        if loop is None:
            if self._loop is None or self._loop.is_closed():
                self._loop = asyncio.new_event_loop()
                self._loop_owner = True
            asyncio.set_event_loop(self._loop)
            loop = self._loop
        
        # 运行协程
        try:
            return loop.run_until_complete(coro)
        except RuntimeError as e:
            # 如果事件循环出现问题，尝试重新创建
            if "Event loop is closed" in str(e) or "This event loop is already running" in str(e):
                if self._loop_owner and self._loop:
                    try:
                        self._loop.close()
                    except:
                        pass
                self._loop = asyncio.new_event_loop()
                self._loop_owner = True
                asyncio.set_event_loop(self._loop)
                return self._loop.run_until_complete(coro)
            raise
    
    # ==================== 经验学习引擎API ====================
    
    def learn(
        self,
        domain: str = "general",
        feedback_cases: Optional[List[Dict[str, Any]]] = None,
        min_case_count: int = 3,
        min_adoption_rate: float = 0.6,
        min_confidence_score: float = 0.5,
        strategy: str = "adaptive",
        user_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """学习生成经验规则（同步版）"""
        return self._run_async(
            self._async_client.learn(
                domain=domain,
                feedback_cases=feedback_cases,
                min_case_count=min_case_count,
                min_adoption_rate=min_adoption_rate,
                min_confidence_score=min_confidence_score,
                strategy=strategy,
                user_id=user_id
            )
        )
    
    def recall(
        self,
        context: Dict[str, Any],
        domain: str = "general",
        top_k: int = 5,
        min_confidence: float = 0.5,
        user_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """召回相关经验规则（同步版）"""
        return self._run_async(
            self._async_client.recall(
                context=context,
                domain=domain,
                top_k=top_k,
                min_confidence=min_confidence,
                user_id=user_id
            )
        )
    
    def batch_learn(
        self,
        domains: List[str],
        min_case_count: int = 3,
        min_adoption_rate: float = 0.6,
        min_confidence_score: float = 0.5,
        strategy: str = "adaptive"
    ) -> List[Dict[str, Any]]:
        """批量学习（同步版）"""
        return self._run_async(
            self._async_client.batch_learn(
                domains=domains,
                min_case_count=min_case_count,
                min_adoption_rate=min_adoption_rate,
                min_confidence_score=min_confidence_score,
                strategy=strategy
            )
        )
    
    # ==================== 记忆管理API ====================
    
    def add_memory(
        self,
        user_id: int,
        domain: str,
        content: Dict[str, Any],
        layer_type: str = "L1",
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """添加记忆（同步版）"""
        return self._run_async(
            self._async_client.add_memory(
                user_id=user_id,
                domain=domain,
                content=content,
                layer_type=layer_type,
                tags=tags,
                keywords=keywords
            )
        )
    
    def search_memory(
        self,
        user_id: Optional[int] = None,
        domain: Optional[str] = None,
        layer_type: Optional[str] = None,
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        limit: int = 20,
        offset: int = 0
    ) -> Dict[str, Any]:
        """搜索记忆（同步版）"""
        return self._run_async(
            self._async_client.search_memory(
                user_id=user_id,
                domain=domain,
                layer_type=layer_type,
                tags=tags,
                keywords=keywords,
                limit=limit,
                offset=offset
            )
        )
    
    def get_memory(self, memory_id: str) -> Dict[str, Any]:
        """获取记忆（同步版）"""
        return self._run_async(self._async_client.get_memory(memory_id))
    
    def update_memory(
        self,
        memory_id: str,
        content: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """更新记忆（同步版）"""
        return self._run_async(
            self._async_client.update_memory(
                memory_id=memory_id,
                content=content,
                tags=tags,
                keywords=keywords
            )
        )
    
    def delete_memory(self, memory_id: str) -> Dict[str, Any]:
        """删除记忆（同步版）"""
        return self._run_async(self._async_client.delete_memory(memory_id))
    
    def batch_add_memories(
        self,
        memories: List[Dict[str, Any]]
    ) -> List[Dict[str, Any]]:
        """批量添加记忆（同步版）"""
        return self._run_async(
            self._async_client.batch_add_memories(memories)
        )
    
    def generate_memory(
        self,
        character_id: str,
        session_id: str,
        messages: List[Dict[str, Any]],
        user_id: Optional[str] = None,
        format: str = "compact"
    ) -> List[Dict[str, Any]]:
        """
        生成记忆（同步版）- 使用 compact 格式
        
        Args:
            character_id: 角色ID（expert_id）
            session_id: 会话ID
            messages: 对话消息列表
            user_id: 用户ID（可选）
            format: 响应格式，默认为 "compact"
        
        Returns:
            compact格式响应数组
        """
        return self._run_async(
            self._async_client.generate_memory(
                character_id=character_id,
                session_id=session_id,
                messages=messages,
                user_id=user_id,
                format=format
            )
        )
    
    def search_memories(
        self,
        query_text: str,
        user_id: Optional[str] = None,
        character_id: Optional[str] = None,
        include_context: bool = False,
        format: str = "simple",
        search_mode: str = "enhanced_semantic",
        score_threshold: float = 0.5
    ) -> Dict[str, Any]:
        """
        智能搜索记忆（同步版）- 使用 simple 格式
        
        Args:
            query_text: 查询文本（语义搜索）
            user_id: 用户ID（可选）
            character_id: 角色ID（可选）
            include_context: 是否包含上下文信息
            format: 响应格式，默认为 "simple"
            search_mode: 搜索模式，默认为 "enhanced_semantic"
            score_threshold: 相似度分数阈值，默认 0.5
        
        Returns:
            simple格式响应对象
        """
        return self._run_async(
            self._async_client.search_memories(
                query_text=query_text,
                user_id=user_id,
                character_id=character_id,
                include_context=include_context,
                format=format,
                search_mode=search_mode,
                score_threshold=score_threshold
            )
        )
    
    # ==================== 用户画像API ====================
    
    def compute_profile(
        self,
        user_id: int,
        domain: str,
        source_memory_ids: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """计算用户画像（同步版）"""
        return self._run_async(
            self._async_client.compute_profile(
                user_id=user_id,
                domain=domain,
                source_memory_ids=source_memory_ids
            )
        )
    
    def get_profile(
        self,
        user_id: int,
        domain: str
    ) -> Dict[str, Any]:
        """获取用户画像（同步版）"""
        return self._run_async(
            self._async_client.get_profile(user_id=user_id, domain=domain)
        )
    
    def search_users(
        self,
        criteria: Dict[str, Any],
        domain: Optional[str] = None,
        limit: int = 20
    ) -> Dict[str, Any]:
        """搜索用户（同步版）"""
        return self._run_async(
            self._async_client.search_users(
                criteria=criteria,
                domain=domain,
                limit=limit
            )
        )
    
    # ==================== 系统API ====================
    
    def health_check(self) -> Dict[str, Any]:
        """健康检查（同步版）"""
        return self._run_async(self._async_client.health_check())
    
    def get_system_status(self) -> Dict[str, Any]:
        """获取系统状态（同步版）"""
        return self._run_async(self._async_client.get_system_status())
    
    def get_health_status(self) -> Dict[str, Any]:
        """获取健康状态（同步版）"""
        return self._run_async(self._async_client.get_health_status())
    
    def get_client_stats(self) -> Dict[str, Any]:
        """获取客户端统计（同步版）"""
        return self._async_client.get_client_stats()
    
    def reset_connections(self):
        """重置连接（同步版）"""
        return self._run_async(self._async_client.reset_connections())
    
    def reset_circuit_breaker(self):
        """重置熔断器（同步版）"""
        return self._async_client.reset_circuit_breaker()
    
    def close(self):
        """
        关闭客户端连接
        
        清理资源，关闭事件循环和异步客户端。
        """
        # 关闭异步客户端
        if self._async_client:
            try:
                if self._loop and not self._loop.is_closed():
                    self._loop.run_until_complete(self._async_client.close())
            except Exception:
                pass
        
        # 关闭事件循环
        if self._loop_owner and self._loop and not self._loop.is_closed():
            try:
                self._loop.close()
            except Exception:
                pass
        
        self._loop = None
        self._loop_owner = False
    
    def __del__(self):
        """析构函数，确保资源释放"""
        try:
            self.close()
        except Exception:
            pass


# 便利函数
def create_sync_client(
    api_key: str,
    base_url: str = "http://localhost:8001",
    enable_enhanced_features: bool = True,
    **kwargs
) -> TiMEMClient:
    """
    创建同步TiMEM客户端
    
    Args:
        api_key: TiMEM API密钥
        base_url: API基础URL
        enable_enhanced_features: 是否启用增强功能
        **kwargs: 其他参数
    
    Returns:
        同步TiMEM客户端
    """
    return TiMEMClient(
        api_key=api_key,
        base_url=base_url,
        enable_connection_pool=enable_enhanced_features,
        enable_circuit_breaker=enable_enhanced_features,
        enable_monitoring=enable_enhanced_features,
        **kwargs
    )



"""
TiMEM监控和日志组件

提供企业级监控、日志记录、性能指标收集等功能。
"""

import asyncio
import time
import logging
import json
import uuid
from typing import Dict, Any, Optional, List, Callable
from dataclasses import dataclass, asdict
from datetime import datetime
from contextlib import asynccontextmanager
from .exceptions import TiMEMError


@dataclass
class RequestMetrics:
    """请求指标"""
    request_id: str
    method: str
    endpoint: str
    start_time: float
    end_time: Optional[float] = None
    duration: Optional[float] = None
    status_code: Optional[int] = None
    success: Optional[bool] = None
    error_message: Optional[str] = None
    user_id: Optional[str] = None
    domain: Optional[str] = None


@dataclass
class SystemMetrics:
    """系统指标"""
    timestamp: float
    cpu_usage: float
    memory_usage: float
    active_connections: int
    total_requests: int
    successful_requests: int
    failed_requests: int
    average_response_time: float
    error_rate: float


class StructuredLogger:
    """结构化日志记录器"""
    
    def __init__(self, name: str, level: int = logging.INFO):
        self.logger = logging.getLogger(name)
        self.logger.setLevel(level)
        
        # 创建JSON格式化器
        if not self.logger.handlers:
            handler = logging.StreamHandler()
            formatter = logging.Formatter(
                '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            )
            handler.setFormatter(formatter)
            self.logger.addHandler(handler)
    
    def log_request(
        self,
        request_id: str,
        method: str,
        endpoint: str,
        duration: float,
        status_code: int,
        success: bool,
        user_id: Optional[str] = None,
        domain: Optional[str] = None,
        error_message: Optional[str] = None
    ):
        """记录请求日志"""
        log_data = {
            "type": "request",
            "request_id": request_id,
            "method": method,
            "endpoint": endpoint,
            "duration": duration,
            "status_code": status_code,
            "success": success,
            "timestamp": datetime.utcnow().isoformat(),
            "user_id": user_id,
            "domain": domain
        }
        
        if error_message:
            log_data["error_message"] = error_message
        
        if success:
            self.logger.info(json.dumps(log_data))
        else:
            self.logger.error(json.dumps(log_data))
    
    def log_system_event(
        self,
        event_type: str,
        message: str,
        data: Optional[Dict[str, Any]] = None
    ):
        """记录系统事件日志"""
        log_data = {
            "type": "system_event",
            "event_type": event_type,
            "message": message,
            "timestamp": datetime.utcnow().isoformat()
        }
        
        if data:
            log_data.update(data)
        
        self.logger.info(json.dumps(log_data))
    
    def log_business_event(
        self,
        event_type: str,
        user_id: str,
        domain: str,
        data: Optional[Dict[str, Any]] = None
    ):
        """记录业务事件日志"""
        log_data = {
            "type": "business_event",
            "event_type": event_type,
            "user_id": user_id,
            "domain": domain,
            "timestamp": datetime.utcnow().isoformat()
        }
        
        if data:
            log_data.update(data)
        
        self.logger.info(json.dumps(log_data))


class MetricsCollector:
    """指标收集器"""
    
    def __init__(self):
        self.metrics: List[RequestMetrics] = []
        self.system_metrics: List[SystemMetrics] = []
        self.max_metrics_history = 1000
        
        # 统计信息
        self.total_requests = 0
        self.successful_requests = 0
        self.failed_requests = 0
        self.total_duration = 0.0
        
        # 日志记录器
        self.logger = StructuredLogger(f"{__name__}.MetricsCollector")
    
    def record_request(
        self,
        method: str,
        endpoint: str,
        duration: float,
        status_code: int,
        success: bool,
        user_id: Optional[str] = None,
        domain: Optional[str] = None,
        error_message: Optional[str] = None
    ):
        """记录请求指标"""
        request_id = str(uuid.uuid4())
        
        metric = RequestMetrics(
            request_id=request_id,
            method=method,
            endpoint=endpoint,
            start_time=time.time() - duration,
            end_time=time.time(),
            duration=duration,
            status_code=status_code,
            success=success,
            error_message=error_message,
            user_id=user_id,
            domain=domain
        )
        
        # 添加到历史记录
        self.metrics.append(metric)
        if len(self.metrics) > self.max_metrics_history:
            self.metrics.pop(0)
        
        # 更新统计信息
        self.total_requests += 1
        if success:
            self.successful_requests += 1
        else:
            self.failed_requests += 1
        self.total_duration += duration
        
        # 记录日志
        self.logger.log_request(
            request_id=request_id,
            method=method,
            endpoint=endpoint,
            duration=duration,
            status_code=status_code,
            success=success,
            user_id=user_id,
            domain=domain,
            error_message=error_message
        )
    
    def record_system_metrics(
        self,
        cpu_usage: float,
        memory_usage: float,
        active_connections: int
    ):
        """记录系统指标"""
        system_metric = SystemMetrics(
            timestamp=time.time(),
            cpu_usage=cpu_usage,
            memory_usage=memory_usage,
            active_connections=active_connections,
            total_requests=self.total_requests,
            successful_requests=self.successful_requests,
            failed_requests=self.failed_requests,
            average_response_time=self.get_average_response_time(),
            error_rate=self.get_error_rate()
        )
        
        self.system_metrics.append(system_metric)
        if len(self.system_metrics) > self.max_metrics_history:
            self.system_metrics.pop(0)
    
    def get_average_response_time(self) -> float:
        """获取平均响应时间"""
        if self.total_requests == 0:
            return 0.0
        return self.total_duration / self.total_requests
    
    def get_error_rate(self) -> float:
        """获取错误率"""
        if self.total_requests == 0:
            return 0.0
        return self.failed_requests / self.total_requests
    
    def get_success_rate(self) -> float:
        """获取成功率"""
        if self.total_requests == 0:
            return 0.0
        return self.successful_requests / self.total_requests
    
    def get_recent_metrics(self, limit: int = 100) -> List[RequestMetrics]:
        """获取最近的指标"""
        return self.metrics[-limit:] if self.metrics else []
    
    def get_system_metrics_summary(self) -> Dict[str, Any]:
        """获取系统指标摘要"""
        if not self.system_metrics:
            return {}
        
        latest = self.system_metrics[-1]
        return {
            "timestamp": latest.timestamp,
            "cpu_usage": latest.cpu_usage,
            "memory_usage": latest.memory_usage,
            "active_connections": latest.active_connections,
            "total_requests": latest.total_requests,
            "successful_requests": latest.successful_requests,
            "failed_requests": latest.failed_requests,
            "average_response_time": latest.average_response_time,
            "error_rate": latest.error_rate,
            "success_rate": self.get_success_rate()
        }


class PerformanceMonitor:
    """性能监控器"""
    
    def __init__(self, name: str):
        self.name = name
        self.metrics_collector = MetricsCollector()
        self.logger = StructuredLogger(f"{__name__}.{name}")
        
        # 监控配置
        self.slow_request_threshold = 5.0  # 慢请求阈值（秒）
        self.high_error_rate_threshold = 0.1  # 高错误率阈值
        
        # 告警回调
        self.alert_callbacks: List[Callable] = []
    
    def add_alert_callback(self, callback: Callable):
        """添加告警回调"""
        self.alert_callbacks.append(callback)
    
    def _check_alerts(self, metric: RequestMetrics):
        """检查告警条件"""
        # 慢请求告警
        if metric.duration and metric.duration > self.slow_request_threshold:
            self._trigger_alert("slow_request", {
                "request_id": metric.request_id,
                "duration": metric.duration,
                "threshold": self.slow_request_threshold
            })
        
        # 高错误率告警
        if self.metrics_collector.get_error_rate() > self.high_error_rate_threshold:
            self._trigger_alert("high_error_rate", {
                "error_rate": self.metrics_collector.get_error_rate(),
                "threshold": self.high_error_rate_threshold
            })
    
    def _trigger_alert(self, alert_type: str, data: Dict[str, Any]):
        """触发告警"""
        alert_data = {
            "type": "alert",
            "alert_type": alert_type,
            "monitor_name": self.name,
            "timestamp": datetime.utcnow().isoformat(),
            "data": data
        }
        
        self.logger.logger.warning(json.dumps(alert_data))
        
        # 调用告警回调
        for callback in self.alert_callbacks:
            try:
                callback(alert_type, data)
            except Exception as e:
                self.logger.logger.error(f"告警回调执行失败: {str(e)}")
    
    def record_request(
        self,
        method: str,
        endpoint: str,
        duration: float,
        status_code: int,
        success: bool,
        user_id: Optional[str] = None,
        domain: Optional[str] = None,
        error_message: Optional[str] = None
    ):
        """记录请求并检查告警"""
        self.metrics_collector.record_request(
            method=method,
            endpoint=endpoint,
            duration=duration,
            status_code=status_code,
            success=success,
            user_id=user_id,
            domain=domain,
            error_message=error_message
        )
        
        # 创建指标对象用于告警检查
        metric = RequestMetrics(
            request_id=str(uuid.uuid4()),
            method=method,
            endpoint=endpoint,
            start_time=time.time() - duration,
            end_time=time.time(),
            duration=duration,
            status_code=status_code,
            success=success,
            error_message=error_message,
            user_id=user_id,
            domain=domain
        )
        
        self._check_alerts(metric)
    
    def get_health_status(self) -> Dict[str, Any]:
        """获取健康状态"""
        summary = self.metrics_collector.get_system_metrics_summary()
        
        # 健康状态判断
        is_healthy = True
        issues = []
        
        if summary.get("error_rate", 0) > self.high_error_rate_threshold:
            is_healthy = False
            issues.append(f"错误率过高: {summary['error_rate']:.2%}")
        
        if summary.get("average_response_time", 0) > self.slow_request_threshold:
            is_healthy = False
            issues.append(f"平均响应时间过长: {summary['average_response_time']:.2f}s")
        
        return {
            "is_healthy": is_healthy,
            "issues": issues,
            "summary": summary,
            "thresholds": {
                "slow_request": self.slow_request_threshold,
                "high_error_rate": self.high_error_rate_threshold
            }
        }


class MonitoringDecorator:
    """监控装饰器"""
    
    def __init__(self, monitor: PerformanceMonitor):
        self.monitor = monitor
    
    def __call__(self, func):
        """装饰器实现"""
        if asyncio.iscoroutinefunction(func):
            async def async_wrapper(*args, **kwargs):
                start_time = time.time()
                success = True
                error_message = None
                status_code = 200
                
                try:
                    result = await func(*args, **kwargs)
                    return result
                except Exception as e:
                    success = False
                    error_message = str(e)
                    status_code = 500
                    raise
                finally:
                    duration = time.time() - start_time
                    self.monitor.record_request(
                        method="POST",  # 默认方法
                        endpoint=func.__name__,
                        duration=duration,
                        status_code=status_code,
                        success=success,
                        error_message=error_message
                    )
            
            return async_wrapper
        else:
            def sync_wrapper(*args, **kwargs):
                start_time = time.time()
                success = True
                error_message = None
                status_code = 200
                
                try:
                    result = func(*args, **kwargs)
                    return result
                except Exception as e:
                    success = False
                    error_message = str(e)
                    status_code = 500
                    raise
                finally:
                    duration = time.time() - start_time
                    self.monitor.record_request(
                        method="POST",  # 默认方法
                        endpoint=func.__name__,
                        duration=duration,
                        status_code=status_code,
                        success=success,
                        error_message=error_message
                    )
            
            return sync_wrapper


# 全局监控器实例
_global_monitor = PerformanceMonitor("TiMEM_Global")


def get_global_monitor() -> PerformanceMonitor:
    """获取全局监控器"""
    return _global_monitor


def monitor_function(monitor: Optional[PerformanceMonitor] = None):
    """函数监控装饰器"""
    if monitor is None:
        monitor = _global_monitor
    
    return MonitoringDecorator(monitor)



"""
TiMEM Python SDK

A comprehensive Time-based Memory Management and Experience Learning toolkit.
"""

__version__ = "0.1.1"
__author__ = "AIGility Cloud Innovation"
__email__ = "contact@aigility.com"
__description__ = "Time-based Memory Management and Experience Learning toolkit"

# 导入同步和异步客户端
from .sync_client import TiMEMClient, create_sync_client
from .async_client import TiMEMClient as AsyncTiMEMClient, learn_async, recall_async, create_client
from .connection_pool import ConnectionPool, ConnectionConfig, get_connection_pool
from .circuit_breaker import CircuitBreaker, CircuitBreakerConfig, get_circuit_breaker
from .monitoring import PerformanceMonitor, get_global_monitor
from .memory import Memory
from .exceptions import (
    TiMEMError, AuthenticationError, APIError, ValidationError,
    ConnectionError, CircuitBreakerError, TimeoutError, RateLimitError
)

__all__ = [
    # 简化的记忆管理接口（推荐使用）
    "Memory",
    
    # 主客户端（同步，简单易用）
    "TiMEMClient",
    "create_sync_client",
    
    # 异步客户端（高性能）
    "AsyncTiMEMClient",
    "learn_async",
    "recall_async",
    "create_client",
    
    # 连接池
    "ConnectionPool",
    "ConnectionConfig",
    "get_connection_pool",
    
    # 熔断器
    "CircuitBreaker",
    "CircuitBreakerConfig",
    "get_circuit_breaker",
    
    # 监控
    "PerformanceMonitor",
    "get_global_monitor",
    
    # 异常
    "TiMEMError", 
    "AuthenticationError",
    "APIError",
    "ValidationError",
    "ConnectionError",
    "CircuitBreakerError",
    "TimeoutError",
    "RateLimitError",
    
    # 元信息
    "__version__",
    "__author__",
    "__email__",
    "__description__",
]



"""
TiMEM SDK 客户端测试

基础单元测试示例
"""

import pytest
from timem import TiMEMClient, AsyncTiMEMClient
from timem.exceptions import ValidationError, AuthenticationError


class TestTiMEMClient:
    """同步客户端测试"""
    
    def test_client_initialization(self):
        """测试客户端初始化"""
        client = TiMEMClient(
            api_key="test-key",
            base_url="http://localhost:8001"
        )
        assert client.api_key == "test-key"
        assert client.base_url == "http://localhost:8001"
        client.close()
    
    def test_client_requires_api_key(self):
        """测试客户端需要 API Key"""
        with pytest.raises(ValidationError):
            TiMEMClient(api_key="")
    
    def test_client_context_manager(self):
        """测试上下文管理器"""
        with TiMEMClient(api_key="test-key") as client:
            assert client is not None
    
    def test_learn_requires_domain(self):
        """测试 learn 需要 domain"""
        client = TiMEMClient(api_key="test-key")
        with pytest.raises(ValidationError):
            client.learn(domain="")
        client.close()
    
    def test_recall_requires_context(self):
        """测试 recall 需要 context"""
        client = TiMEMClient(api_key="test-key")
        with pytest.raises(ValidationError):
            client.recall(context={})
        client.close()
    
    def test_add_memory_requires_content(self):
        """测试 add_memory 需要 content"""
        client = TiMEMClient(api_key="test-key")
        with pytest.raises(ValidationError):
            client.add_memory(
                user_id=123,
                domain="test",
                content=None
            )
        client.close()


class TestAsyncTiMEMClient:
    """异步客户端测试"""
    
    def test_async_client_initialization(self):
        """测试异步客户端初始化"""
        client = AsyncTiMEMClient(
            api_key="test-key",
            base_url="http://localhost:8001"
        )
        assert client.api_key == "test-key"
        assert client.base_url == "http://localhost:8001"
    
    def test_async_client_requires_api_key(self):
        """测试异步客户端需要 API Key"""
        with pytest.raises(ValidationError):
            AsyncTiMEMClient(api_key="")
    
    @pytest.mark.asyncio
    async def test_async_client_context_manager(self):
        """测试异步上下文管理器"""
        async with AsyncTiMEMClient(api_key="test-key") as client:
            assert client is not None
    
    @pytest.mark.asyncio
    async def test_async_learn_requires_domain(self):
        """测试异步 learn 需要 domain"""
        async with AsyncTiMEMClient(api_key="test-key") as client:
            with pytest.raises(ValidationError):
                await client.learn(domain="")
    
    @pytest.mark.asyncio
    async def test_async_recall_requires_context(self):
        """测试异步 recall 需要 context"""
        async with AsyncTiMEMClient(api_key="test-key") as client:
            with pytest.raises(ValidationError):
                await client.recall(context={})
    
    @pytest.mark.asyncio
    async def test_async_batch_learn(self):
        """测试批量学习"""
        async with AsyncTiMEMClient(api_key="test-key") as client:
            # 测试空列表
            results = await client.batch_learn(domains=[])
            assert results == []
    
    @pytest.mark.asyncio
    async def test_async_batch_add_memories(self):
        """测试批量添加记忆"""
        async with AsyncTiMEMClient(api_key="test-key") as client:
            # 测试空列表
            results = await client.batch_add_memories(memories=[])
            assert results == []


class TestConvenienceFunctions:
    """便捷函数测试"""
    
    @pytest.mark.asyncio
    async def test_learn_async(self):
        """测试便捷学习函数"""
        from timem import learn_async
        
        # 测试需要 API key
        with pytest.raises(ValidationError):
            await learn_async(api_key="", domain="test")
    
    @pytest.mark.asyncio
    async def test_recall_async(self):
        """测试便捷召回函数"""
        from timem import recall_async
        
        # 测试需要 API key
        with pytest.raises(ValidationError):
            await recall_async(api_key="", context={"test": "test"})


# 集成测试标记
@pytest.mark.integration
class TestIntegration:
    """集成测试（需要运行的 TiMEM Engine）"""
    
    def test_health_check(self):
        """测试健康检查"""
        # 跳过，因为需要实际的服务
        pytest.skip("Requires running TiMEM Engine")
    
    @pytest.mark.asyncio
    async def test_async_health_check(self):
        """测试异步健康检查"""
        # 跳过，因为需要实际的服务
        pytest.skip("Requires running TiMEM Engine")


if __name__ == "__main__":
    pytest.main([__file__, "-v"])




"""
TiMEM连接池管理器

提供企业级连接池管理，支持连接复用、健康检查、自动重连等功能。
"""

import asyncio
import time
import logging
from typing import Dict, Any, Optional, List
from dataclasses import dataclass
from contextlib import asynccontextmanager
import httpx
from .exceptions import TiMEMError, ConnectionError


@dataclass
class ConnectionConfig:
    """连接配置"""
    max_connections: int = 20
    max_keepalive_connections: int = 10
    keepalive_expiry: float = 30.0  # httpx使用keepalive_expiry而不是keepalive_timeout
    connect_timeout: float = 10.0
    read_timeout: float = 60.0
    write_timeout: float = 60.0
    retry_attempts: int = 3
    retry_delay: float = 1.0
    health_check_interval: float = 30.0
    max_retries: int = 3


class ConnectionPool:
    """连接池管理器"""
    
    def __init__(
        self,
        base_url: str,
        api_key: str,
        config: Optional[ConnectionConfig] = None,
        verify_ssl: bool = False
    ):
        self.base_url = base_url.rstrip('/')
        self.api_key = api_key
        self.config = config or ConnectionConfig()
        self.verify_ssl = verify_ssl
        
        # 连接池状态
        self._pool: Optional[httpx.AsyncClient] = None
        self._last_health_check = 0
        self._is_healthy = True
        self._connection_count = 0
        self._failed_requests = 0
        self._successful_requests = 0
        
        # 日志记录器
        self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
        
        # 健康检查任务
        self._health_check_task: Optional[asyncio.Task] = None
    
    async def __aenter__(self):
        """异步上下文管理器入口"""
        await self.initialize()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器出口"""
        await self.close()
    
    async def initialize(self):
        """初始化连接池"""
        if self._pool is None:
            await self._create_pool()
            await self._start_health_check()
    
    async def _create_pool(self):
        """创建连接池"""
        try:
            headers = {
                "accept": "application/json",
                "X-API-Key": self.api_key,  # TiMEM Engine 使用 X-API-Key 头
                "Content-Type": "application/json",
                "User-Agent": f"timem-python-pool/{__import__('timem').__version__}"
            }
            
            limits = httpx.Limits(
                max_connections=self.config.max_connections,
                max_keepalive_connections=self.config.max_keepalive_connections,
                keepalive_expiry=self.config.keepalive_expiry
            )
            
            timeout = httpx.Timeout(
                connect=self.config.connect_timeout,
                read=self.config.read_timeout,
                write=self.config.write_timeout,
                pool=None  # httpx要求所有4个参数或default参数
            )
            
            self._pool = httpx.AsyncClient(
                base_url=self.base_url,
                headers=headers,
                limits=limits,
                timeout=timeout,
                verify=self.verify_ssl
            )
            
            self._connection_count = 0
            self.logger.info(f"连接池已创建: {self.base_url}")
            
        except Exception as e:
            self.logger.error(f"创建连接池失败: {str(e)}")
            raise ConnectionError(f"Failed to create connection pool: {str(e)}")
    
    async def _start_health_check(self):
        """启动健康检查任务"""
        if self._health_check_task is None:
            self._health_check_task = asyncio.create_task(self._health_check_loop())
    
    async def _health_check_loop(self):
        """健康检查循环"""
        while self._pool is not None:
            try:
                await asyncio.sleep(self.config.health_check_interval)
                await self._perform_health_check()
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.logger.warning(f"健康检查失败: {str(e)}")
                self._is_healthy = False
    
    async def _perform_health_check(self):
        """执行健康检查"""
        if self._pool is None:
            return
        
        try:
            start_time = time.time()
            response = await self._pool.get("/api/v1/health", timeout=5.0)
            
            if response.status_code == 200:
                self._is_healthy = True
                self._last_health_check = time.time()
                self.logger.debug(f"健康检查通过: {time.time() - start_time:.2f}s")
            else:
                self._is_healthy = False
                self.logger.warning(f"健康检查失败: HTTP {response.status_code}")
                
        except Exception as e:
            self._is_healthy = False
            self.logger.warning(f"健康检查异常: {str(e)}")
    
    async def get_connection(self) -> httpx.AsyncClient:
        """获取连接"""
        if self._pool is None:
            await self.initialize()
        
        if not self._is_healthy:
            await self._perform_health_check()
        
        if not self._is_healthy:
            raise ConnectionError("连接池不健康")
        
        return self._pool
    
    async def make_request(
        self,
        method: str,
        endpoint: str,
        data: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
        **kwargs
    ) -> Dict[str, Any]:
        """使用连接池发送请求"""
        client = await self.get_connection()
        
        for attempt in range(self.config.retry_attempts):
            try:
                start_time = time.time()
                response = await client.request(
                    method=method,
                    url=endpoint,
                    json=data,
                    params=params,
                    **kwargs
                )
                
                # 记录成功请求
                self._successful_requests += 1
                self.logger.debug(
                    f"请求成功: {method} {endpoint} "
                    f"({time.time() - start_time:.2f}s)"
                )
                
                # 处理响应
                if response.status_code >= 400:
                    self._failed_requests += 1
                    try:
                        error_data = response.json()
                        error_message = error_data.get('message', f'HTTP {response.status_code} error')
                    except:
                        error_message = f'HTTP {response.status_code} error'
                    raise httpx.HTTPStatusError(error_message, request=response.request, response=response)
                
                return response.json()
                
            except (httpx.RequestError, httpx.HTTPStatusError) as e:
                self._failed_requests += 1
                
                if attempt == self.config.retry_attempts - 1:
                    self.logger.error(f"请求失败 (尝试 {attempt + 1}/{self.config.retry_attempts}): {str(e)}")
                    raise
                
                # 等待重试
                await asyncio.sleep(self.config.retry_delay * (attempt + 1))
                self.logger.warning(f"请求重试 {attempt + 1}/{self.config.retry_attempts}: {str(e)}")
    
    def get_stats(self) -> Dict[str, Any]:
        """获取连接池统计信息"""
        return {
            "base_url": self.base_url,
            "is_healthy": self._is_healthy,
            "last_health_check": self._last_health_check,
            "connection_count": self._connection_count,
            "successful_requests": self._successful_requests,
            "failed_requests": self._failed_requests,
            "success_rate": (
                self._successful_requests / (self._successful_requests + self._failed_requests)
                if (self._successful_requests + self._failed_requests) > 0 else 0
            ),
            "config": {
                "max_connections": self.config.max_connections,
                "max_keepalive_connections": self.config.max_keepalive_connections,
                "health_check_interval": self.config.health_check_interval
            }
        }
    
    async def close(self):
        """关闭连接池"""
        if self._health_check_task:
            self._health_check_task.cancel()
            try:
                await self._health_check_task
            except asyncio.CancelledError:
                pass
        
        if self._pool:
            await self._pool.aclose()
            self._pool = None
        
        self.logger.info("连接池已关闭")
    
    async def reset(self):
        """重置连接池"""
        await self.close()
        await self.initialize()
        self.logger.info("连接池已重置")


class ConnectionPoolManager:
    """连接池管理器 - 支持多实例管理"""
    
    def __init__(self):
        self._pools: Dict[str, ConnectionPool] = {}
        self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
    
    async def get_pool(
        self,
        base_url: str,
        api_key: str,
        config: Optional[ConnectionConfig] = None,
        verify_ssl: bool = False
    ) -> ConnectionPool:
        """获取或创建连接池"""
        pool_key = f"{base_url}:{api_key}"
        
        if pool_key not in self._pools:
            pool = ConnectionPool(
                base_url=base_url,
                api_key=api_key,
                config=config,
                verify_ssl=verify_ssl
            )
            await pool.initialize()
            self._pools[pool_key] = pool
            self.logger.info(f"创建新连接池: {base_url}")
        
        return self._pools[pool_key]
    
    async def close_all(self):
        """关闭所有连接池"""
        for pool in self._pools.values():
            await pool.close()
        self._pools.clear()
        self.logger.info("所有连接池已关闭")
    
    def get_all_stats(self) -> Dict[str, Any]:
        """获取所有连接池统计信息"""
        return {
            pool_key: pool.get_stats()
            for pool_key, pool in self._pools.items()
        }


# 全局连接池管理器实例
_global_pool_manager = ConnectionPoolManager()


async def get_connection_pool(
    base_url: str,
    api_key: str,
    config: Optional[ConnectionConfig] = None,
    verify_ssl: bool = False
) -> ConnectionPool:
    """获取全局连接池"""
    return await _global_pool_manager.get_pool(
        base_url=base_url,
        api_key=api_key,
        config=config,
        verify_ssl=verify_ssl
    )


async def close_all_pools():
    """关闭所有全局连接池"""
    await _global_pool_manager.close_all()



"""
TiMEM Python SDK Client

Main client class for interacting with the TiMEM API.
"""

import httpx
from typing import Optional, Dict, Any, List
from .exceptions import TiMEMError, AuthenticationError, APIError, ValidationError


class TiMEMClient:
    """
    Main client for interacting with the TiMEM API.
    
    This client provides methods for:
    - Learning from feedback (learn)
    - Recalling rules (recall)
    - Adding memories (add_memory)
    - Searching memories (search_memory)
    """
    
    def __init__(
        self,
        api_key: str,
        base_url: str = "http://localhost:8001",
        timeout: float = 60.0,
        verify_ssl: bool = False,  # 默认不验证SSL，简化使用
        **kwargs
    ):
        """
        Initialize the TiMEM client.
        
        Args:
            api_key: Your TiMEM API key
            base_url: Base URL for the API (default: http://localhost:8001)
            timeout: Request timeout in seconds (default: 60.0)
            verify_ssl: Whether to verify SSL certificates (default: False)
            **kwargs: Additional arguments passed to httpx.Client
        """
        if not api_key:
            raise ValidationError("API key is required")
        
        self.api_key = api_key
        self.base_url = base_url.rstrip('/')
        self.timeout = timeout
        
        # Set up default headers
        headers = {
            "accept": "application/json",
            "Authorization": f"Bearer {api_key}",
            "Content-Type": "application/json",
            "User-Agent": f"timem-python/{__import__('timem').__version__}"
        }
        
        # Create httpx client with configurable SSL verification
        self.client = httpx.Client(
            base_url=self.base_url,
            headers=headers,
            timeout=timeout,
            verify=verify_ssl,  # 使用verify_ssl参数
            **kwargs
        )
    
    def __enter__(self):
        """Context manager entry."""
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit."""
        self.close()
    
    def close(self):
        """Close the HTTP client."""
        if hasattr(self, 'client'):
            self.client.close()
    
    def _make_request(
        self,
        method: str,
        endpoint: str,
        data: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
        **kwargs
    ) -> Dict[str, Any]:
        """
        Make an HTTP request to the API.
        
        Args:
            method: HTTP method (GET, POST, PUT, DELETE)
            endpoint: API endpoint
            data: Request body data
            params: Query parameters
            **kwargs: Additional arguments for httpx
            
        Returns:
            Response data as dictionary
            
        Raises:
            AuthenticationError: If authentication fails
            APIError: If API returns an error
            TiMEMError: For other client errors
        """
        try:
            response = self.client.request(
                method=method,
                url=endpoint,
                json=data,
                params=params,
                **kwargs
            )
            
            # Handle authentication errors
            if response.status_code == 401:
                raise AuthenticationError("Invalid API key or authentication failed")
            
            # Handle other HTTP errors
            if response.status_code >= 400:
                try:
                    error_data = response.json()
                    error_message = error_data.get('message', error_data.get('detail', f'HTTP {response.status_code} error'))
                except:
                    error_message = f'HTTP {response.status_code} error'
                raise APIError(error_message, status_code=response.status_code)
            
            # Return JSON response
            return response.json()
            
        except httpx.RequestError as e:
            raise TiMEMError(f"Request failed: {str(e)}")
        except httpx.HTTPStatusError as e:
            if e.response.status_code == 401:
                raise AuthenticationError("Invalid API key or authentication failed")
            raise APIError(f"HTTP {e.response.status_code} error: {e.response.text}")
    
    # ==================== Experience Learning Engine APIs ====================
    
    def learn(
        self,
        domain: str = "general",
        feedback_cases: Optional[List[Dict[str, Any]]] = None,
        min_case_count: int = 3,
        min_adoption_rate: float = 0.6,
        min_confidence_score: float = 0.5,
        strategy: str = "adaptive"
    ) -> Dict[str, Any]:
        """
        Learn from feedback cases to generate experience rules.
        
        Uses LLM to analyze feedback cases and generate actionable rules.
        No clustering - direct LLM call for more intelligent rule generation.
        
        If feedback_cases provided, will first collect them then trigger learning.
        If not provided, will learn from existing unprocessed cases in the system.
        
        Args:
            domain: Business domain (e.g., "aicv", "education", "general")
            feedback_cases: Optional feedback cases to collect before learning
            min_case_count: Minimum number of cases required to generate a rule
            min_adoption_rate: Minimum adoption rate threshold (0-1)
            min_confidence_score: Minimum confidence score threshold (0-1)
            strategy: Summarization strategy ("single", "cluster", "adaptive")
            
        Returns:
            Learning result with generated rules
        """
        if not domain:
            raise ValidationError("Domain is required")
        
        # If feedback cases provided, collect them first
        if feedback_cases:
            import datetime
            batch_id = f"batch_{datetime.datetime.now().strftime('%Y%m%d_%H%M%S')}"
            
            collect_data = {
                "batch_id": batch_id,
                "feedback_cases": feedback_cases
            }
            
            # Collect feedback
            self._make_request("POST", "/api/v1/rules/feedback/collect", data=collect_data)
        
        # Trigger learning
        data = {
            "domain": domain,
            "min_case_count": min_case_count,
            "min_adoption_rate": min_adoption_rate,
            "min_confidence_score": min_confidence_score
        }
        
        params = {"summarization_strategy": strategy}
        
        return self._make_request("POST", "/api/v1/rules/learn-with-llm", data=data, params=params)
    
    def recall(
        self,
        context: Dict[str, Any],
        domain: str = "general",
        top_k: int = 5,
        min_confidence: float = 0.5
    ) -> Dict[str, Any]:
        """
        Recall relevant experience rules based on context.
        
        Args:
            context: Context for rule matching (e.g., job_title, issue_type, section)
            domain: Business domain
            top_k: Number of top rules to return
            min_confidence: Minimum confidence threshold
            
        Returns:
            Recalled rules with relevance scores
        """
        if not context:
            raise ValidationError("Context is required")
        
        data = {
            "domain": domain,
            "context": context,
            "top_k": top_k,
            "min_confidence": min_confidence
        }
        
        return self._make_request("POST", "/api/v1/rules/rule-call", data=data)
    
    # ==================== Memory Management APIs ====================
    
    def add_memory(
        self,
        user_id: int,
        domain: str,
        content: Dict[str, Any],
        layer_type: str = "L1",
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        Add a memory to the TiMEM system.
        
        Args:
            user_id: User identifier
            domain: Business domain
            content: Memory content
            layer_type: Memory layer (L1-L5)
            tags: Optional tags for categorization
            keywords: Optional keywords for search
            
        Returns:
            Created memory information
        """
        if not content:
            raise ValidationError("Memory content is required")
        
        data = {
            "user_id": user_id,
            "domain": domain,
            "content": content,
            "layer_type": layer_type,
            "tags": tags or [],
            "keywords": keywords or []
        }
        
        return self._make_request("POST", "/api/v1/memory/memories", data=data)
    
    def search_memory(
        self,
        user_id: Optional[int] = None,
        domain: Optional[str] = None,
        layer_type: Optional[str] = None,
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        limit: int = 20,
        offset: int = 0
    ) -> Dict[str, Any]:
        """
        Search memories in the TiMEM system.
        
        Args:
            user_id: Optional user filter
            domain: Optional domain filter
            layer_type: Optional layer type filter
            tags: Optional tags filter
            keywords: Optional keywords filter
            limit: Maximum number of results
            offset: Pagination offset
            
        Returns:
            Search results with memories
        """
        params = {
            "limit": limit,
            "offset": offset
        }
        
        if user_id:
            params["user_id"] = user_id
        if domain:
            params["domain"] = domain
        if layer_type:
            params["layer_type"] = layer_type
        if tags:
            params["tags"] = ",".join(tags)
        if keywords:
            params["keywords"] = ",".join(keywords)
        
        return self._make_request("GET", "/api/v1/memory/memories", params=params)
    
    def get_memory(self, memory_id: str) -> Dict[str, Any]:
        """
        Get a specific memory by ID.
        
        Args:
            memory_id: Memory identifier
            
        Returns:
            Memory information
        """
        if not memory_id:
            raise ValidationError("Memory ID is required")
        
        return self._make_request("GET", f"/api/v1/memory/memories/{memory_id}")
    
    def update_memory(
        self,
        memory_id: str,
        content: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        Update an existing memory.
        
        Args:
            memory_id: Memory identifier
            content: Updated content
            tags: Updated tags
            keywords: Updated keywords
            
        Returns:
            Update result
        """
        if not memory_id:
            raise ValidationError("Memory ID is required")
        
        data = {}
        if content is not None:
            data["content"] = content
        if tags is not None:
            data["tags"] = tags
        if keywords is not None:
            data["keywords"] = keywords
        
        return self._make_request("PUT", f"/api/v1/memory/memories/{memory_id}", data=data)
    
    def delete_memory(self, memory_id: str) -> Dict[str, Any]:
        """
        Delete a memory.
        
        Args:
            memory_id: Memory identifier
            
        Returns:
            Deletion result
        """
        if not memory_id:
            raise ValidationError("Memory ID is required")
        
        return self._make_request("DELETE", f"/api/v1/memory/memories/{memory_id}")
    
    # ==================== User Profile APIs ====================
    
    def compute_profile(
        self,
        user_id: int,
        domain: str,
        source_memory_ids: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        Compute user profile from L5 level memories.
        
        Args:
            user_id: User identifier
            domain: Business domain
            source_memory_ids: Optional specific memory IDs to use
            
        Returns:
            Computed profile information
        """
        data = {
            "user_id": user_id,
            "domain": domain
        }
        
        if source_memory_ids:
            data["source_memory_ids"] = source_memory_ids
        
        return self._make_request("POST", "/api/v1/user-profile/compute", data=data)
    
    def get_profile(
        self,
        user_id: int,
        domain: str
    ) -> Dict[str, Any]:
        """
        Get user profile.
        
        Args:
            user_id: User identifier
            domain: Business domain
            
        Returns:
            User profile information
        """
        params = {
            "user_id": user_id,
            "domain": domain
        }
        
        return self._make_request("GET", "/api/v1/user-profile/profiles", params=params)
    
    def search_users(
        self,
        criteria: Dict[str, Any],
        domain: Optional[str] = None,
        limit: int = 20
    ) -> Dict[str, Any]:
        """
        Search users by profile criteria.
        
        Args:
            criteria: Search criteria (e.g., preferences, behavior patterns)
            domain: Optional domain filter
            limit: Maximum number of results
            
        Returns:
            Matching users
        """
        if not criteria:
            raise ValidationError("Search criteria is required")
        
        data = {
            "criteria": criteria,
            "limit": limit
        }
        
        if domain:
            data["domain"] = domain
        
        return self._make_request("POST", "/api/v1/user-profile/search", data=data)
    
    # ==================== System APIs ====================
    
    def health_check(self) -> Dict[str, Any]:
        """
        Check the health of the TiMEM API.
        
        Returns:
            Health status
        """
        try:
            response = self.client.get("/api/v1/health")
            return response.json()
        except Exception as e:
            return {
                "status": "unhealthy",
                "error": str(e)
            }
    
    def get_system_status(self) -> Dict[str, Any]:
        """
        Get system status and module information.
        
        Returns:
            System status
        """
        return self._make_request("GET", "/api/v1/status")



"""
Compatibility alias package for end-users to `import memory` instead of `import timem`.

This package re-exports the public API from the `timem` package.
Users can use `import memory` as an alias to `import timem`.
"""

# Import all public symbols from timem
from timem import *

# Explicitly import metadata and re-export them
from timem import (
    __version__,
    __author__,
    __email__,
    __description__,
    __all__ as _timem_all__,
)

# Use the same __all__ as timem to maintain consistency
__all__ = _timem_all__






"""
TiMEM SDK Command Line Interface

Simple CLI tool for TiMEM operations.
"""

import sys
import json
import argparse
from typing import Optional
from .client import TiMEMClient
from .exceptions import TiMEMError


def create_client(api_key: str, base_url: str) -> TiMEMClient:
    """Create TiMEM client."""
    return TiMEMClient(api_key=api_key, base_url=base_url)


def cmd_health(args):
    """Check TiMEM service health."""
    client = create_client(args.api_key, args.base_url)
    try:
        result = client.health_check()
        print(json.dumps(result, indent=2, ensure_ascii=False))
    except TiMEMError as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)
    finally:
        client.close()


def cmd_learn(args):
    """Trigger learning from feedback."""
    client = create_client(args.api_key, args.base_url)
    try:
        result = client.learn(
            domain=args.domain,
            min_case_count=args.min_cases,
            min_adoption_rate=args.min_adoption,
            strategy=args.strategy
        )
        print(json.dumps(result, indent=2, ensure_ascii=False))
    except TiMEMError as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)
    finally:
        client.close()


def cmd_recall(args):
    """Recall rules by context."""
    client = create_client(args.api_key, args.base_url)
    try:
        # Parse context from JSON string
        context = json.loads(args.context)
        
        result = client.recall(
            context=context,
            domain=args.domain,
            top_k=args.top_k,
            min_confidence=args.min_confidence
        )
        print(json.dumps(result, indent=2, ensure_ascii=False))
    except json.JSONDecodeError as e:
        print(f"Error parsing context JSON: {e}", file=sys.stderr)
        sys.exit(1)
    except TiMEMError as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)
    finally:
        client.close()


def cmd_add_memory(args):
    """Add a memory."""
    client = create_client(args.api_key, args.base_url)
    try:
        # Parse content from JSON string
        content = json.loads(args.content)
        tags = args.tags.split(',') if args.tags else None
        
        result = client.add_memory(
            user_id=args.user_id,
            domain=args.domain,
            content=content,
            layer_type=args.layer,
            tags=tags
        )
        print(json.dumps(result, indent=2, ensure_ascii=False))
    except json.JSONDecodeError as e:
        print(f"Error parsing content JSON: {e}", file=sys.stderr)
        sys.exit(1)
    except TiMEMError as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)
    finally:
        client.close()


def cmd_search_memory(args):
    """Search memories."""
    client = create_client(args.api_key, args.base_url)
    try:
        tags = args.tags.split(',') if args.tags else None
        
        result = client.search_memory(
            user_id=args.user_id if args.user_id else None,
            domain=args.domain if args.domain else None,
            tags=tags,
            limit=args.limit
        )
        print(json.dumps(result, indent=2, ensure_ascii=False))
    except TiMEMError as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)
    finally:
        client.close()


def main():
    """Main CLI entry point."""
    parser = argparse.ArgumentParser(
        description="TiMEM SDK Command Line Interface",
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    
    parser.add_argument(
        '--api-key',
        required=True,
        help='TiMEM API key'
    )
    parser.add_argument(
        '--base-url',
        default='http://localhost:8001',
        help='TiMEM Engine base URL (default: http://localhost:8001)'
    )
    
    subparsers = parser.add_subparsers(dest='command', help='Available commands')
    
    # Health command
    parser_health = subparsers.add_parser('health', help='Check service health')
    parser_health.set_defaults(func=cmd_health)
    
    # Learn command
    parser_learn = subparsers.add_parser('learn', help='Trigger learning from feedback')
    parser_learn.add_argument('--domain', default='general', help='Business domain')
    parser_learn.add_argument('--min-cases', type=int, default=3, help='Minimum case count')
    parser_learn.add_argument('--min-adoption', type=float, default=0.6, help='Minimum adoption rate')
    parser_learn.add_argument('--strategy', default='adaptive', choices=['single', 'cluster', 'adaptive'], help='Summarization strategy')
    parser_learn.set_defaults(func=cmd_learn)
    
    # Recall command
    parser_recall = subparsers.add_parser('recall', help='Recall rules by context')
    parser_recall.add_argument('--context', required=True, help='Context JSON string')
    parser_recall.add_argument('--domain', default='general', help='Business domain')
    parser_recall.add_argument('--top-k', type=int, default=5, help='Number of top rules')
    parser_recall.add_argument('--min-confidence', type=float, default=0.5, help='Minimum confidence')
    parser_recall.set_defaults(func=cmd_recall)
    
    # Add memory command
    parser_add = subparsers.add_parser('add-memory', help='Add a memory')
    parser_add.add_argument('--user-id', type=int, required=True, help='User ID')
    parser_add.add_argument('--domain', required=True, help='Business domain')
    parser_add.add_argument('--content', required=True, help='Memory content JSON string')
    parser_add.add_argument('--layer', default='L1', help='Memory layer (L1-L5)')
    parser_add.add_argument('--tags', help='Comma-separated tags')
    parser_add.set_defaults(func=cmd_add_memory)
    
    # Search memory command
    parser_search = subparsers.add_parser('search-memory', help='Search memories')
    parser_search.add_argument('--user-id', type=int, help='User ID filter')
    parser_search.add_argument('--domain', help='Domain filter')
    parser_search.add_argument('--tags', help='Comma-separated tags filter')
    parser_search.add_argument('--limit', type=int, default=20, help='Result limit')
    parser_search.set_defaults(func=cmd_search_memory)
    
    args = parser.parse_args()
    
    if not args.command:
        parser.print_help()
        sys.exit(1)
    
    args.func(args)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name="timem-ai",
    version="0.0.1",
    description="TiMEM Python SDK - Time-based Memory Management and Experience Learning toolkit",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="AIGility Cloud Innovation",
    author_email="contact@aigility.com",
    url="https://github.com/AIGility-Cloud-Innovation/timem-python",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Database :: Database Engines/Servers",
    ],
    python_requires=">=3.8",
    install_requires=[
        "httpx>=0.24.0",
        "pydantic>=2.0.0",
        "typing-extensions>=4.0.0",
    ],
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "pytest-asyncio>=0.21.0",
            "pytest-cov>=4.0.0",
            "black>=22.0.0",
            "isort>=5.0.0",
            "flake8>=5.0.0",
            "mypy>=1.0.0",
            "pre-commit>=2.20.0",
        ],
        "docs": [
            "sphinx>=5.0.0",
            "sphinx-rtd-theme>=1.0.0",
            "myst-parser>=0.18.0",
        ],
    },
    entry_points={
        "console_scripts": [
            "timem=timem.cli:main",
        ],
    },
    include_package_data=True,
    zip_safe=False,
)



"""
TiMEM Python SDK Async Client

Async client class for interacting with the TiMEM API with full async support.
"""

import asyncio
import httpx
import time
import logging
from typing import Optional, Dict, Any, List
from .exceptions import TiMEMError, AuthenticationError, APIError, ValidationError, ConnectionError, CircuitBreakerError
from .connection_pool import ConnectionPool, ConnectionConfig, get_connection_pool
from .circuit_breaker import CircuitBreaker, CircuitBreakerConfig, get_circuit_breaker
from .monitoring import PerformanceMonitor, get_global_monitor


class TiMEMClient:
    """
    TiMEM API client with enterprise-grade features.
    
    This client provides methods for:
    - Learning from feedback (learn)
    - Recalling rules (recall)
    - Adding memories (add_memory)
    - Searching memories (search_memory)
    - Connection pooling, circuit breaker, monitoring
    """
    
    def __init__(
        self,
        api_key: str,
        base_url: str = "http://localhost:8001",
        timeout: float = 60.0,
        max_retries: int = 3,
        retry_delay: float = 1.0,
        verify_ssl: bool = False,  # 默认不验证SSL，简化使用
        # 增强功能参数
        enable_connection_pool: bool = True,
        enable_circuit_breaker: bool = True,
        enable_monitoring: bool = True,
        connection_config: Optional[ConnectionConfig] = None,
        circuit_breaker_config: Optional[CircuitBreakerConfig] = None,
        **kwargs
    ):
        """
        Initialize the TiMEM client.
        
        Args:
            api_key: Your TiMEM API key
            base_url: Base URL for the API (default: http://localhost:8001)
            timeout: Request timeout in seconds (default: 60.0)
            max_retries: Maximum number of retries for failed requests (default: 3)
            retry_delay: Delay between retries in seconds (default: 1.0)
            verify_ssl: Whether to verify SSL certificates (default: False)
            enable_connection_pool: Enable connection pooling (default: True)
            enable_circuit_breaker: Enable circuit breaker (default: True)
            enable_monitoring: Enable performance monitoring (default: True)
            connection_config: Connection pool configuration
            circuit_breaker_config: Circuit breaker configuration
            **kwargs: Additional arguments passed to httpx.AsyncClient
        """
        if not api_key:
            raise ValidationError("API key is required")
        
        self.api_key = api_key
        self.base_url = base_url.rstrip('/')
        self.timeout = timeout
        self.max_retries = max_retries
        self.retry_delay = retry_delay
        self.verify_ssl = verify_ssl
        
        # 保存配置供后续使用
        self._connection_config = connection_config
        self._circuit_breaker_config = circuit_breaker_config
        
        # 增强功能配置
        self.enable_connection_pool = enable_connection_pool
        self.enable_circuit_breaker = enable_circuit_breaker
        self.enable_monitoring = enable_monitoring
        
        # 连接池和熔断器（延迟初始化）
        self._pool: Optional[ConnectionPool] = None
        self._circuit_breaker: Optional[CircuitBreaker] = None
        self._monitor: Optional[PerformanceMonitor] = None
        
        # 统计信息
        self.stats = {
            "total_requests": 0,
            "successful_requests": 0,
            "failed_requests": 0,
            "circuit_breaker_trips": 0,
            "connection_pool_resets": 0
        }
        
        # 日志记录器
        self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
        
        # 传统httpx客户端（作为备用）
        headers = {
            "accept": "application/json",
            "X-API-Key": api_key,  # TiMEM Engine 使用 X-API-Key 头
            "Content-Type": "application/json",
            "User-Agent": f"timem-python-async/{__import__('timem').__version__}"
        }
        
        self.client = httpx.AsyncClient(
            base_url=self.base_url,
            headers=headers,
            timeout=timeout,
            verify=verify_ssl,
            **kwargs
        )
    
    async def __aenter__(self):
        """Async context manager entry."""
        await self._initialize_enhanced_features()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Async context manager exit."""
        await self.close()
    
    async def _initialize_enhanced_features(self):
        """初始化增强功能"""
        try:
            # 初始化连接池
            if self.enable_connection_pool:
                self._pool = await get_connection_pool(
                    base_url=self.base_url,
                    api_key=self.api_key,
                    config=self._connection_config,
                    verify_ssl=self.verify_ssl
                )
                self.logger.info("连接池已初始化")
            
            # 初始化熔断器
            if self.enable_circuit_breaker:
                self._circuit_breaker = get_circuit_breaker(
                    name=f"timem_{self.base_url}",
                    config=self._circuit_breaker_config,
                    fallback_func=self._fallback_handler
                )
                self.logger.info("熔断器已初始化")
            
            # 初始化监控
            if self.enable_monitoring:
                self._monitor = get_global_monitor()
                self.logger.info("监控已初始化")
                
        except Exception as e:
            self.logger.warning(f"增强功能初始化失败: {str(e)}")
    
    async def _fallback_handler(self, *args, **kwargs) -> Dict[str, Any]:
        """降级处理函数"""
        self.stats["circuit_breaker_trips"] += 1
        self.logger.warning("熔断器触发，使用降级处理")
        
        return {
            "success": False,
            "error": "Service temporarily unavailable",
            "fallback": True,
            "message": "TiMEM服务暂时不可用，请稍后重试"
        }
    
    async def close(self):
        """Close the async HTTP client and release all resources."""
        # 关闭连接池
        if self._pool:
            await self._pool.close()
            self._pool = None
        
        # 关闭httpx客户端
        if hasattr(self, 'client') and self.client is not None:
            await self.client.aclose()
            self.client = None
        
        self.logger.info("TiMEM客户端已关闭")
    
    async def _make_request(
        self,
        method: str,
        endpoint: str,
        data: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
        user_id: Optional[str] = None,
        domain: Optional[str] = None,
        **kwargs
    ) -> Dict[str, Any]:
        """
        Make an async HTTP request to the API with enhanced features.
        
        Args:
            method: HTTP method (GET, POST, PUT, DELETE)
            endpoint: API endpoint
            data: Request body data
            params: Query parameters
            user_id: User ID for monitoring
            domain: Domain for monitoring
            **kwargs: Additional arguments for httpx
            
        Returns:
            Response data as dictionary
            
        Raises:
            AuthenticationError: If authentication fails
            APIError: If API returns an error
            TiMEMError: For other client errors
        """
        start_time = time.time()
        success = False
        error_message = None
        status_code = 200
        
        try:
            # 使用增强功能
            if self._pool and self._circuit_breaker:
                # 使用连接池和熔断器
                result = await self._circuit_breaker.call(
                    self._pool.make_request,
                    method=method,
                    endpoint=endpoint,
                    data=data,
                    params=params,
                    **kwargs
                )
                success = True
                self.stats["successful_requests"] += 1
                return result
            else:
                # 使用传统方式
                return await self._make_traditional_request(
                    method, endpoint, data, params, **kwargs
                )
                
        except CircuitBreakerError as e:
            error_message = f"Circuit breaker open: {str(e)}"
            status_code = 503
            self.stats["circuit_breaker_trips"] += 1
            raise
            
        except (ConnectionError, APIError, AuthenticationError) as e:
            error_message = str(e)
            status_code = getattr(e, 'status_code', 500)
            self.stats["failed_requests"] += 1
            raise
            
        except Exception as e:
            error_message = str(e)
            status_code = 500
            self.stats["failed_requests"] += 1
            raise TiMEMError(f"Request failed: {str(e)}")
            
        finally:
            # 记录统计信息
            self.stats["total_requests"] += 1
            duration = time.time() - start_time
            
            # 记录监控数据
            if self._monitor:
                self._monitor.record_request(
                    method=method,
                    endpoint=endpoint,
                    duration=duration,
                    status_code=status_code,
                    success=success,
                    user_id=user_id,
                    domain=domain,
                    error_message=error_message
                )
    
    async def _make_traditional_request(
        self,
        method: str,
        endpoint: str,
        data: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
        **kwargs
    ) -> Dict[str, Any]:
        """传统请求方式（备用）"""
        last_exception = None
        
        for attempt in range(self.max_retries + 1):
            try:
                response = await self.client.request(
                    method=method,
                    url=endpoint,
                    json=data,
                    params=params,
                    **kwargs
                )
                
                # Handle authentication errors
                if response.status_code == 401:
                    raise AuthenticationError("Invalid API key or authentication failed")
                
                # Handle other HTTP errors
                if response.status_code >= 400:
                    try:
                        error_data = response.json()
                        error_message = error_data.get('message', error_data.get('detail', f'HTTP {response.status_code} error'))
                    except:
                        error_message = f'HTTP {response.status_code} error'
                    raise APIError(error_message, status_code=response.status_code)
                
                # Return JSON response
                return response.json()
                
            except (httpx.RequestError, httpx.HTTPStatusError, APIError, AuthenticationError) as e:
                last_exception = e
                
                # Don't retry authentication errors
                if isinstance(e, AuthenticationError):
                    raise e
                
                # Don't retry on last attempt
                if attempt == self.max_retries:
                    break
                
                # Wait before retry
                await asyncio.sleep(self.retry_delay * (attempt + 1))
        
        # Re-raise the last exception
        if isinstance(last_exception, httpx.RequestError):
            raise TiMEMError(f"Request failed after {self.max_retries + 1} attempts: {str(last_exception)}")
        elif isinstance(last_exception, httpx.HTTPStatusError):
            raise APIError(f"HTTP {last_exception.response.status_code} error after {self.max_retries + 1} attempts: {last_exception.response.text}")
        else:
            raise last_exception
    
    # ==================== Experience Learning Engine APIs ====================
    
    async def learn(
        self,
        domain: str = "general",
        feedback_cases: Optional[List[Dict[str, Any]]] = None,
        min_case_count: int = 3,
        min_adoption_rate: float = 0.6,
        min_confidence_score: float = 0.5,
        strategy: str = "adaptive",
        user_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        Learn from feedback cases to generate experience rules.
        
        Uses LLM to analyze feedback cases and generate actionable rules.
        No clustering - direct LLM call for more intelligent rule generation.
        
        If feedback_cases provided, will first collect them then trigger learning.
        If not provided, will learn from existing unprocessed cases in the system.
        
        Args:
            domain: Business domain (e.g., "aicv", "education", "general")
            feedback_cases: Optional feedback cases to collect before learning
            min_case_count: Minimum number of cases required to generate a rule
            min_adoption_rate: Minimum adoption rate threshold (0-1)
            min_confidence_score: Minimum confidence score threshold (0-1)
            strategy: Summarization strategy ("single", "cluster", "adaptive")
            user_id: Optional user identifier for monitoring
            
        Returns:
            Learning result with generated rules
        """
        if not domain:
            raise ValidationError("Domain is required")
        
        # If feedback cases provided, collect them first
        if feedback_cases:
            import datetime
            batch_id = f"batch_{datetime.datetime.now().strftime('%Y%m%d_%H%M%S')}"
            
            collect_data = {
                "batch_id": batch_id,
                "feedback_cases": feedback_cases
            }
            
            # Collect feedback
            await self._make_request("POST", "/api/v1/rules/feedback/collect", data=collect_data)
        
        # Trigger learning
        data = {
            "domain": domain,
            "min_case_count": min_case_count,
            "min_adoption_rate": min_adoption_rate,
            "min_confidence_score": min_confidence_score
        }
        
        params = {"summarization_strategy": strategy}
        
        return await self._make_request("POST", "/api/v1/rules/learn-with-llm", data=data, params=params, user_id=user_id, domain=domain)
    
    async def recall(
        self,
        context: Dict[str, Any],
        domain: str = "general",
        top_k: int = 5,
        min_confidence: float = 0.5,
        user_id: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        Recall relevant experience rules based on context.
        
        Args:
            context: Context for rule matching (e.g., job_title, issue_type, section)
            domain: Business domain
            top_k: Number of top rules to return
            min_confidence: Minimum confidence threshold
            user_id: Optional user identifier for monitoring
            
        Returns:
            Recalled rules with relevance scores
        """
        if not context:
            raise ValidationError("Context is required")
        
        data = {
            "domain": domain,
            "context": context,
            "top_k": top_k,
            "min_confidence": min_confidence
        }
        
        return await self._make_request("POST", "/api/v1/rules/rule-call", data=data, user_id=user_id, domain=domain)
    
    async def batch_learn(
        self,
        domains: List[str],
        min_case_count: int = 3,
        min_adoption_rate: float = 0.6,
        min_confidence_score: float = 0.5,
        strategy: str = "adaptive"
    ) -> List[Dict[str, Any]]:
        """
        Learn from feedback cases for multiple domains concurrently.
        
        Args:
            domains: List of business domains
            min_case_count: Minimum number of cases required to generate a rule
            min_adoption_rate: Minimum adoption rate threshold (0-1)
            min_confidence_score: Minimum confidence score threshold (0-1)
            strategy: Summarization strategy
            
        Returns:
            List of learning results for each domain
        """
        if not domains:
            return []
        
        # Create tasks for concurrent execution
        tasks = []
        for domain in domains:
            task = self.learn(
                domain=domain,
                min_case_count=min_case_count,
                min_adoption_rate=min_adoption_rate,
                min_confidence_score=min_confidence_score,
                strategy=strategy
            )
            tasks.append(task)
        
        # Execute all tasks concurrently
        try:
            results = await asyncio.gather(*tasks, return_exceptions=True)
            
            # Process results
            processed_results = []
            for i, result in enumerate(results):
                if isinstance(result, Exception):
                    processed_results.append({
                        'error': str(result),
                        'success': False,
                        'domain': domains[i]
                    })
                else:
                    processed_results.append({
                        'result': result,
                        'success': True,
                        'domain': domains[i]
                    })
            
            return processed_results
            
        except Exception as e:
            raise TiMEMError(f"Batch learn failed: {str(e)}")
    
    # ==================== Memory Management APIs ====================
    
    async def add_memory(
        self,
        user_id: int,
        domain: str,
        content: Dict[str, Any],
        layer_type: str = "L1",
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        Add a memory to the TiMEM system.
        
        Args:
            user_id: User identifier
            domain: Business domain
            content: Memory content (will be converted to JSON string)
            layer_type: Memory layer (L1-L5)
            tags: Optional tags for categorization
            keywords: Optional keywords for search
            
        Returns:
            Created memory information
        """
        if not content:
            raise ValidationError("Memory content is required")
        
        # 将content转换为字符串格式（API期望字符串类型）
        import json
        content_str = json.dumps(content, ensure_ascii=False) if isinstance(content, dict) else str(content)
        
        # 确保domain是有效值：aicv, education, consulting, customer_service, general
        valid_domains = ['aicv', 'education', 'consulting', 'customer_service', 'general']
        if domain not in valid_domains:
            domain = 'general'  # 默认使用general
        
        # 注意：TiMEM Engine的API中，memory_type和layer使用相同的值（L1-L5）
        data = {
            "user_id": str(user_id),  # API期望字符串类型
            "layer": layer_type,  # 记忆层级
            "memory_type": layer_type,  # memory_type也使用相同的值（L1-L5）
            "content": content_str,
            "domain": domain,
            "tags": tags or [],
            "keywords": keywords or []
        }
        
        # 调试日志
        self.logger.debug(f"[add_memory] 发送数据: {json.dumps(data, ensure_ascii=False, indent=2)}")
        
        return await self._make_request("POST", "/api/v1/memory/create", data=data)
    
    async def search_memory(
        self,
        user_id: Optional[int] = None,
        domain: Optional[str] = None,
        layer_type: Optional[str] = None,
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        limit: int = 20,
        offset: int = 0
    ) -> Dict[str, Any]:
        """
        Search memories in the TiMEM system.
        
        Args:
            user_id: Optional user filter
            domain: Optional domain filter
            layer_type: Optional layer type filter
            tags: Optional tags filter
            keywords: Optional keywords filter
            limit: Maximum number of results
            offset: Pagination offset
            
        Returns:
            Search results with memories
        """
        params = {
            "limit": limit,
            "offset": offset
        }
        
        if user_id:
            params["user_id"] = user_id
        if domain:
            params["domain"] = domain
        if layer_type:
            params["layer_type"] = layer_type
        if tags:
            params["tags"] = ",".join(tags)
        if keywords:
            params["keywords"] = ",".join(keywords)
        
        return await self._make_request("GET", "/api/v1/memory/query", params=params)
    
    async def get_memory(self, memory_id: str) -> Dict[str, Any]:
        """
        Get a specific memory by ID.
        
        Args:
            memory_id: Memory identifier
            
        Returns:
            Memory information
        """
        if not memory_id:
            raise ValidationError("Memory ID is required")
        
        return await self._make_request("GET", f"/api/v1/memory/{memory_id}")
    
    async def update_memory(
        self,
        memory_id: str,
        content: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        Update an existing memory.
        
        Args:
            memory_id: Memory identifier
            content: Updated content
            tags: Updated tags
            keywords: Updated keywords
            
        Returns:
            Update result
        """
        if not memory_id:
            raise ValidationError("Memory ID is required")
        
        data = {}
        if content is not None:
            data["content"] = content
        if tags is not None:
            data["tags"] = tags
        if keywords is not None:
            data["keywords"] = keywords
        
        return await self._make_request("PUT", f"/api/v1/memory/{memory_id}", data=data)
    
    async def delete_memory(self, memory_id: str) -> Dict[str, Any]:
        """
        Delete a memory.
        
        Args:
            memory_id: Memory identifier
            
        Returns:
            Deletion result
        """
        if not memory_id:
            raise ValidationError("Memory ID is required")
        
        return await self._make_request("DELETE", f"/api/v1/memory/{memory_id}")
    
    async def batch_add_memories(
        self,
        memories: List[Dict[str, Any]]
    ) -> List[Dict[str, Any]]:
        """
        Add multiple memories concurrently.
        
        Args:
            memories: List of memory dictionaries (user_id, domain, content, etc.)
            
        Returns:
            List of creation results
        """
        if not memories:
            return []
        
        # Create tasks for concurrent execution
        tasks = []
        for mem in memories:
            task = self.add_memory(
                user_id=mem.get('user_id'),
                domain=mem.get('domain'),
                content=mem.get('content'),
                layer_type=mem.get('layer_type', 'L1'),
                tags=mem.get('tags'),
                keywords=mem.get('keywords')
            )
            tasks.append(task)
        
        # Execute all tasks concurrently
        try:
            results = await asyncio.gather(*tasks, return_exceptions=True)
            
            # Process results
            processed_results = []
            for i, result in enumerate(results):
                if isinstance(result, Exception):
                    processed_results.append({
                        'error': str(result),
                        'success': False,
                        'index': i
                    })
                else:
                    processed_results.append({
                        'result': result,
                        'success': True,
                        'index': i
                    })
            
            return processed_results
            
        except Exception as e:
            raise TiMEMError(f"Batch add memories failed: {str(e)}")
    
    # ==================== 对话记忆生成和智能搜索 API ====================
    
    async def generate_memory(
        self,
        character_id: str,
        session_id: str,
        messages: List[Dict[str, Any]],
        user_id: Optional[str] = None,
        format: str = "compact"
    ) -> List[Dict[str, Any]]:
        """
        生成记忆 - 使用 compact 格式（简化响应）
        
        从对话消息中自动提取并生成记忆。
        
        Args:
            character_id: 角色ID（expert_id）
            session_id: 会话ID
            messages: 对话消息列表 [{"role": "user/assistant", "content": "..."}]
            user_id: 用户ID（可选）
            format: 响应格式，默认为 "compact"
        
        Returns:
            compact格式响应：数组格式
            [
                {
                    "id": "记忆ID",
                    "data": {"memory": "记忆内容"},
                    "event": "ADD"
                }
            ]
        """
        if not character_id:
            raise ValidationError("character_id 必须提供")
        if not session_id:
            raise ValidationError("session_id 必须提供")
        if not messages:
            raise ValidationError("messages 不能为空")
        
        payload = {
            "expert_id": character_id,
            "session_id": session_id,
            "messages": messages,
            "format": format
        }
        if user_id:
            payload["user_id"] = user_id
        
        result = await self._make_request("POST", "/api/v1/memory/", data=payload)
        
        # compact格式返回数组
        if isinstance(result, list):
            return result
        # 如果不是数组，尝试从响应中提取
        if isinstance(result, dict):
            return result.get("data", result.get("memories", [result]))
        return [result] if result else []
    
    async def search_memories(
        self,
        query_text: str,
        user_id: Optional[str] = None,
        character_id: Optional[str] = None,
        include_context: bool = False,
        format: str = "simple",
        search_mode: str = "enhanced_semantic",
        score_threshold: float = 0.5
    ) -> Dict[str, Any]:
        """
        智能搜索记忆 - 使用 simple 格式（简化响应）
        
        基于语义搜索相关记忆。
        
        Args:
            query_text: 查询文本（语义搜索）
            user_id: 用户ID（可选）
            character_id: 角色ID（可选）
            include_context: 是否包含上下文信息（session_id, character_id等）
            format: 响应格式，默认为 "simple"
            search_mode: 搜索模式，默认为 "enhanced_semantic"
            score_threshold: 相似度分数阈值，默认 0.5
        
        Returns:
            simple格式响应：对象格式
            {
                "memories": [
                    {
                        "id": "记忆ID",
                        "memory": "记忆内容",
                        "layer": "L1",
                        "metadata": {
                            "score": 0.85,  # 相似度分数
                            "session_id": "...",  # include_context=true时包含
                            "character_id": "..."  # include_context=true时包含
                        },
                        "created_at": "...",
                        "updated_at": "..."
                    }
                ],
                "total": 10
            }
        """
        if not query_text:
            raise ValidationError("query_text 必须提供")
        
        payload = {
            "query_text": query_text,
            "format": format,
            "include_context": include_context,
            "config": {
                "search_mode": search_mode,
                "score_threshold": score_threshold
            }
        }
        if user_id:
            payload["user_id"] = user_id
        if character_id:
            payload["character_id"] = character_id
        
        result = await self._make_request("POST", "/api/v1/memory/search", data=payload)
        
        # 处理响应
        if result is None:
            return {"memories": [], "total": 0}
        
        # 如果已经是期望的格式，直接返回
        if isinstance(result, dict) and "memories" in result:
            return result
        
        # 尝试转换格式
        if isinstance(result, list):
            return {"memories": result, "total": len(result)}
        
        # 如果返回的是单个对象，包装成列表
        return {"memories": [result] if result else [], "total": 1 if result else 0}
    
    # ==================== User Profile APIs ====================
    
    async def compute_profile(
        self,
        user_id: int,
        domain: str,
        source_memory_ids: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        Compute user profile from L5 level memories.
        
        Args:
            user_id: User identifier
            domain: Business domain
            source_memory_ids: Optional specific memory IDs to use
            
        Returns:
            Computed profile information
        """
        data = {
            "user_id": user_id,
            "domain": domain
        }
        
        if source_memory_ids:
            data["source_memory_ids"] = source_memory_ids
        
        return await self._make_request("POST", "/api/v1/user-profile/compute", data=data)
    
    async def get_profile(
        self,
        user_id: int,
        domain: str
    ) -> Dict[str, Any]:
        """
        Get user profile.
        
        Args:
            user_id: User identifier
            domain: Business domain
            
        Returns:
            User profile information
        """
        params = {
            "user_id": user_id,
            "domain": domain
        }
        
        return await self._make_request("GET", "/api/v1/user-profile/profiles", params=params)
    
    async def search_users(
        self,
        criteria: Dict[str, Any],
        domain: Optional[str] = None,
        limit: int = 20
    ) -> Dict[str, Any]:
        """
        Search users by profile criteria.
        
        Args:
            criteria: Search criteria (e.g., preferences, behavior patterns)
            domain: Optional domain filter
            limit: Maximum number of results
            
        Returns:
            Matching users
        """
        if not criteria:
            raise ValidationError("Search criteria is required")
        
        data = {
            "criteria": criteria,
            "limit": limit
        }
        
        if domain:
            data["domain"] = domain
        
        return await self._make_request("POST", "/api/v1/user-profile/search", data=data)
    
    # ==================== System APIs ====================
    
    async def health_check(self) -> Dict[str, Any]:
        """
        Check the health of the TiMEM API.
        
        Returns:
            Health status
        """
        try:
            response = await self.client.get("/api/v1/health")
            return response.json()
        except Exception as e:
            return {
                "status": "unhealthy",
                "error": str(e)
            }
    
    async def get_system_status(self) -> Dict[str, Any]:
        """
        Get system status and module information.
        
        Returns:
            System status
        """
        return await self._make_request("GET", "/api/v1/status")
    
    # ==================== 增强功能方法 ====================
    
    async def get_health_status(self) -> Dict[str, Any]:
        """
        获取健康状态（增强版）
        """
        try:
            # 基础健康检查
            health_data = await self._make_request("GET", "/api/v1/health")
            
            # 添加增强信息
            enhanced_health = {
                **health_data,
                "enhanced_features": {
                    "connection_pool": self._pool.get_stats() if self._pool else None,
                    "circuit_breaker": self._circuit_breaker.get_state() if self._circuit_breaker else None,
                    "monitoring": self._monitor.get_health_status() if self._monitor else None,
                    "client_stats": self.stats.copy()
                }
            }
            
            return enhanced_health
            
        except Exception as e:
            return {
                "status": "unhealthy",
                "error": str(e),
                "enhanced_features": {
                    "connection_pool": self._pool.get_stats() if self._pool else None,
                    "circuit_breaker": self._circuit_breaker.get_state() if self._circuit_breaker else None,
                    "monitoring": self._monitor.get_health_status() if self._monitor else None,
                    "client_stats": self.stats.copy()
                }
            }
    
    def get_client_stats(self) -> Dict[str, Any]:
        """获取客户端统计信息"""
        return {
            "client_stats": self.stats.copy(),
            "connection_pool": self._pool.get_stats() if self._pool else None,
            "circuit_breaker": self._circuit_breaker.get_state() if self._circuit_breaker else None,
            "monitoring": self._monitor.get_health_status() if self._monitor else None
        }
    
    async def reset_connections(self):
        """重置连接"""
        if self._pool:
            await self._pool.reset()
            self.stats["connection_pool_resets"] += 1
            self.logger.info("连接池已重置")
    
    def reset_circuit_breaker(self):
        """重置熔断器"""
        if self._circuit_breaker:
            self._circuit_breaker.reset()
            self.logger.info("熔断器已重置")


# ==================== Convenience Functions ====================

async def create_client(
    api_key: str,
    base_url: str = "http://localhost:8001",
    enable_enhanced_features: bool = True,
    **kwargs
) -> TiMEMClient:
    """
    创建TiMEM客户端（简化版）
    
    Args:
        api_key: TiMEM API密钥
        base_url: API基础URL
        enable_enhanced_features: 是否启用增强功能
        **kwargs: 其他参数
    
    Returns:
        TiMEM客户端
    """
    if enable_enhanced_features:
        return TiMEMClient(
            api_key=api_key,
            base_url=base_url,
            enable_connection_pool=True,
            enable_circuit_breaker=True,
            enable_monitoring=True,
            **kwargs
        )
    else:
        return TiMEMClient(
            api_key=api_key,
            base_url=base_url,
            enable_connection_pool=False,
            enable_circuit_breaker=False,
            enable_monitoring=False,
            **kwargs
        )


async def learn_async(
    api_key: str,
    domain: str = "general",
    min_case_count: int = 3,
    min_adoption_rate: float = 0.6,
    min_confidence_score: float = 0.5,
    strategy: str = "adaptive",
    **kwargs
) -> Dict[str, Any]:
    """
    Convenience function to learn from feedback asynchronously.
    
    Args:
        api_key: Your TiMEM API key
        domain: Business domain
        min_case_count: Minimum number of cases required
        min_adoption_rate: Minimum adoption rate threshold
        min_confidence_score: Minimum confidence score threshold
        strategy: Summarization strategy
        **kwargs: Additional arguments for AsyncTiMEMClient
        
    Returns:
        Learning result
    """
    async with TiMEMClient(api_key=api_key, **kwargs) as client:
        return await client.learn(
            domain=domain,
            min_case_count=min_case_count,
            min_adoption_rate=min_adoption_rate,
            min_confidence_score=min_confidence_score,
            strategy=strategy
        )


async def recall_async(
    api_key: str,
    context: Dict[str, Any],
    domain: str = "general",
    top_k: int = 5,
    min_confidence: float = 0.5,
    **kwargs
) -> Dict[str, Any]:
    """
    Convenience function to recall rules asynchronously.
    
    Args:
        api_key: Your TiMEM API key
        context: Context for rule matching
        domain: Business domain
        top_k: Number of top rules to return
        min_confidence: Minimum confidence threshold
        **kwargs: Additional arguments for AsyncTiMEMClient
        
    Returns:
        Recalled rules
    """
    async with TiMEMClient(api_key=api_key, **kwargs) as client:
        return await client.recall(
            context=context,
            domain=domain,
            top_k=top_k,
            min_confidence=min_confidence
        )
